/*
 * $Id: //depot/prod/nvpf/Rinxs/src/java/com/netapp/nvpf/api/model/AegisApiService.java#1 $ 
 *
 * Copyright (c) 2013 NetApp, Inc.
 * All rights reserved.
 * 
 */
package com.netapp.nvpf.api.model;

import java.util.Date;
import java.util.List;

import com.netapp.nvpf.api.model.backup.Backup;
import com.netapp.nvpf.api.model.backup.BackupJob;
import com.netapp.nvpf.api.model.backup.BackupJobSummary;
import com.netapp.nvpf.api.model.backup.BackupListResult;
import com.netapp.nvpf.api.model.backup.BackupLocations;
import com.netapp.nvpf.api.model.backup.BackupPolicy;
import com.netapp.nvpf.api.model.backup.BackupRequestFilter;
import com.netapp.nvpf.api.model.backup.Entity;
import com.netapp.nvpf.api.model.backup.ProtectionGroup;
import com.netapp.nvpf.api.model.backup.RestoreInfo;
import com.netapp.nvpf.api.model.backup.RestoreModel;
import com.netapp.nvpf.api.model.backup.configuration.EmailConfiguration;
import com.netapp.nvpf.api.model.backup.configuration.GlobalConfiguration;
import com.netapp.nvpf.api.model.backup.configuration.SnapManagerConfiguration;
import com.netapp.nvpf.api.model.datacenter.Datacenter;
import com.netapp.nvpf.api.model.datastore.Datastore;
import com.netapp.nvpf.api.model.virtualmachine.VirtualMachine;

public interface AegisApiService {

    /**
     * API will call the registerHost to register given configuration details to SMS Server
     * @param snapManagerConfiguration
     * @param nvpfVSphereSession 
     * @throws ApiException
     */
    void registerHost(SnapManagerConfiguration snapManagerConfiguration, NvpfVSphereSession nvpfVSphereSession) throws ApiException;

    /** 
     * API will call the getRegisteredHost to get SMS Server SM configuration details
     * @return SnapManagerConfiguration details.
     * @throws ApiException
     */
    SnapManagerConfiguration getRegisteredHost() throws ApiException;

    /**
     * The method returns the array of policy objects as per the Request query from SnapManager Server.
     * @param requestQuery
     * @return BackupPolicy[]
     * @throws ApiException
     */
    BackupPolicy[] getPolicies(RequestQuery requestQuery) throws ApiException;

    /**
     * The method creates the Protection Group on SnapManager Server.
     * @param session
     * @param protectionGroup
     * @throws ApiException
     */
    void createProtectionGroup(NvpfVSphereSession session, ProtectionGroup protectionGroup) throws ApiException;

    /**
     * Get the count of backups for a specific object
     * @param session Session
     * @param moref Managed Object Reference of the object
     * @return Count of backups
     * @throws ApiException
     */
    
    /**
     * API will suspend the BackupJobs
     * @param Array of protectionGroupIds
     * @throws ApiException
     */
    void suspendProtectionGroups(NvpfVSphereSession session, List<String> protectionGroupIds) throws ApiException;

    /**
     * API will resume the BackupJobs
     * @param Array of protectionGroupIds
     * @throws ApiException
     */
    void resumeProtectionGroups(NvpfVSphereSession session,  List<String> protectionGroupIds) throws ApiException;
    
    int getBackupCount(NvpfVSphereSession session, String moref) throws ApiException;

    /**
     * Get the list of backups for a specific object with a query
     * @param session Session
     * @param moref Managed Object Reference of the object
     * @param requestQuery The query
     * @return List of backups
     * @throws ApiException
     */
    List<Backup> getBackupList(NvpfVSphereSession session, String moref, RequestQuery requestQuery) throws ApiException;

    /**
     * The method deletes Protection Groups on SnapManager Server. 
     * @param session
     * @param backupConfig
     * @throws ApiException
     */
    void deleteProtectionGroups(NvpfVSphereSession session, List<String> protectionGroupIds) throws ApiException;

    /** 
     * Creates a backup from a backup configuration based on a policy.
     * @param session Session
     * @param configName
     * @param policyName
     * @throws ApiException
     */
    void createBackupNow(NvpfVSphereSession session, String configName, String policyName) throws ApiException;

    /** 
     * Api will delete backups based on given ids.
     * @param nvpfVSphereSession 
     * @param backupIds
     * @throws ApiException
     */
    void deleteBackups(NvpfVSphereSession nvpfVSphereSession, List<String> backupIds) throws ApiException;

    /**
     * API will get the Backup job/protection group by id.
     * @param  protectionGroupId
     * @return BackupJob
     * @throws ApiException
     */
    BackupJob getBackupJobById(String backupJobId) throws ApiException;

    /**
     * API will get Backup by id/name
     * @param backupId
     * @return Backup
     * @throws ApiException
     */
    Backup getBackupById(String backupId) throws ApiException;

    /**
     * This method will perform mount operation for selected backup 
     * @param session
     * @param backupId
     * @param hostName
     * @throws ApiException
     */
    void mountBackup(NvpfVSphereSession session, String backupId, String hostName) throws ApiException;

    /**
     * This method will perform mount operation for selected datastore 
     * @param backupId
     * @param datastoreNames
     * @param hostName
     * @throws ApiException
     */
    void mountDatastoreFromBackup(NvpfVSphereSession session, String backupId, List<String> datastoreNames,
        String hostName, List<BackupLocations> backupLocators) throws ApiException;

    /**
     * This method will perform restore operation for selected datastore 
     * @param restoreModel
     * @throws ApiException
     */
    void restore(NvpfVSphereSession session, RestoreModel restoreModel, List<BackupLocations> backupLocators) throws ApiException;

    /**
     * This method will perform restore operation for selected virtualdisks 
     * @param restoreModel
     * @throws ApiException
     */
    void restoreVirtualDisk(NvpfVSphereSession session, RestoreModel restoreModel, List<BackupLocations> backupLocators) throws ApiException;
    /**
     * This method will perform unmount operation for selected datastore 
     * @param backupId
     * @param datastores
     * @throws ApiException
     */
    void unmountDatastores(NvpfVSphereSession session, String backupId, List<Datastore> datastores) throws ApiException;

    /**
     * This method will return the list of datastores mounted from the backup
     * @param backupId
     * @throws ApiException
     */
    List<Datastore> getMountedDatastores(NvpfVSphereSession session, String backupId) throws ApiException;

    /**
     * This api will update the protection group information.
     * @param session
     * @param protectionGroup
     * @throws ApiException
     */
    void updateProtectionGroup(NvpfVSphereSession session, ProtectionGroup protectionGroup) throws ApiException;

    /**
     * API will create the policy on SnapManager Server.
     * @param session
     * @param backupPolicy
     * @throws ApiException
     */
    void addPolicy(NvpfVSphereSession session, BackupPolicy backupPolicy) throws ApiException;

    /**
     * The method will delete policy using parameter policy id.
     * @param session
     * @param policyId
     * @throws ApiException
     */
    void deletePolicy(NvpfVSphereSession session, String policyId) throws ApiException;

    /**
     * The method will get the policy using parameter policy id.
     * @param policyId
     * @throws ApiException
     */
    BackupPolicy getPolicyById(String policyId) throws ApiException;

    /**
     * Gets the information required for the restore GUI
     * @param session vSphere session
     * @param entityUuid UUID of the entity being restores
     * @param entityName Name of the entity being restores
     * @param backupUuid ID of the backup
     * @return Restore information
     * @throws ApiException
     */
    RestoreInfo getRestoreInfo(NvpfVSphereSession session, String entityUuid, String entityName, String backupUuid) throws ApiException;

    /**
     * This method will update the policy.
     * @param session
     * @param backupPolicy
     * @throws ApiException
     */
    void updatePolicy(NvpfVSphereSession session, BackupPolicy backupPolicy) throws ApiException;

    /**
     * This method will fetch the list of filtered backups
     * @param session
     * @param moref
     * @param backupRequestFilter
     * @return
     * @throws ApiException
     */
    BackupListResult getBackupList(NvpfVSphereSession session, String moref, BackupRequestFilter backupRequestFilter) throws ApiException;

    /**
     * This method will check if the passed entities are Snap* compatible.
     * @param managedObjectReferenceList - List of Entities morefs.
     * @param session
     * @return boolean - true if compatible false otherwise.
     * @throws ApiException
     */
    boolean isSnapStarCompatible(List<String> managedObjectReferenceList, NvpfVSphereSession session) throws ApiException;

    /**
     * API will get the list of Backup Jobs from the SnapManager.
     * @param moref - Set the moref/uuid if you want backup Jobs related to particular entity.
     * @param include - Set include true if you want to get list which contains entity with passed moref
     * @param requestQuery
     * @param session
     * @return List<BackupJob>
     * @throws ApiException
     */
    List<BackupJob> getBackupJobList(String moref, boolean include,
        RequestQuery requestQuery, NvpfVSphereSession session) throws ApiException;
    
    
    /**
     * API will get the Count of Backup Jobs from the SnapManager.
     * @param moref - Set the moref/uuid if you want backup Jobs related to particular entity.
     * @param session
     * @return Count of Backup Jobs
     * @throws ApiException
     *
     */
    int getBackupJobCount(String moref, NvpfVSphereSession session) throws ApiException;

    /**
     * API will get the list of ProtectionGroups by moref (property related to DS moref /VM uuid)
     * Set include true if you want to get list which contains entity with passed moref
     * @param moref
     * @param include
     * @param requestQuery
     * @param session
     * @return List<ProtectionGroup>
     * @throws ApiException
     */
    List<ProtectionGroup> getProtectionGroupsByMoref(String moref, boolean include,
        RequestQuery requestQuery, NvpfVSphereSession session) throws ApiException;

    /**
     * API will get the list of DataCenters
     * @param session
     * @return
     * @throws ApiException
     */

    List<Datacenter> getDatacenterList(NvpfVSphereSession session) throws ApiException;

    /**
     * API will send notification Mails based on the configuration details.
     * @param session
     * @param emailConfiguration
     * @return
     * @throws ApiException
     */
    void notificationTestMail(NvpfVSphereSession session, EmailConfiguration emailConfiguration) throws ApiException;

    /**
     * API will rename Backup and its Snapshots based on Id
     * @param nvpfVSphereSession 
     * @param backUpId
     * @param newBackupName
     * @throws ApiException
     */

    void renameBackup(NvpfVSphereSession nvpfVSphereSession, String backUpId, String newBackupName) throws ApiException;

    /**
     * API will get the list of available scripts.
     * @param
     * @return List<String> of scripts.
     * @throws ApiException
     */
    List<String> getAvailableScriptsForPolicy() throws ApiException;

    /**
     * API will get spanned datastore list
     * @param session
     * @param selectedEntities
     * @param includeIndependentDisks
     * @return List<Datastore> of datastore
     * @throws ApiException
     */

    List<Datastore> getSpanningDatastoreList(NvpfVSphereSession session, List<String> selectedEntities) throws ApiException;

    /**
     * The method will get the policy using parameter policy name.
     * @param policyName String
     * @return BackupPolicy.
     * @throws ApiException
     */
    BackupPolicy getPolicyByName(String policyName) throws ApiException;

    /**
     * The method will get the BackupJob using parameter backupJobName.
     * @param backupJobName String
     * @return BackupJob.
     * @throws ApiException
     */
    BackupJob getBackupJobByName(String backupJobName) throws ApiException;
    
    /**
     * The method will get the ProtectionGroup using parameter backupJobName.
     * @param backupJobName String
     * @return ProtectionGroup.
     * @throws ApiException
     */
    ProtectionGroup getProtectionGroupByName(String backupJobName) throws ApiException;


    /**
     * The method will get the entities with moref.
     * @param backupJobName String
     * @return BackupJob.
     * @throws ApiException
     */
    List<Entity> getMorefForEntities(NvpfVSphereSession session, List<Entity> selectedEntityList) throws ApiException;

    /**
     * The method will get the entities with moref.
     * @param aegisBackupNames List of BackupId(s)
     * @return List<Entity> List of Entity(s).
     * @throws ApiException
     */
    List<Entity> getListOfEntityMorefsFromBackup(NvpfVSphereSession session, List<String> aegisBackupNames) throws ApiException;

    /**
     * This method will return Entity Object by using entityMoref
     * @param entityMoref : String
     * @param session : NvpfVSphereSession
     * @return Entity
     * @throws ApiException
     */
    Entity getEntityByMoref(String entityMoref, NvpfVSphereSession session) throws ApiException;
    
    /**
     * This API will fetch datastore volume's primary and secondary locations for a specified backup.
     * @param backupId : String
     * @return List<BackupLocations>
     * @throws ApiException
     */
    List<BackupLocations> getDatastoreBackupLocators(String backupId, NvpfVSphereSession session) throws ApiException;
    
    /**
     * This API will fetch datastore volume's primary and secondary locations for a specified backup based on virtualEntityId and destDatastoreIds passed in restoreModel
     * @param backupId : String
     * @param restoreModel
     * @return List<BackupLocations>
     * @throws ApiException
     */
    List<BackupLocations> getDatastoreBackupLocatorsForRestore(NvpfVSphereSession session, String backupId, RestoreModel restoreModel) throws ApiException;

    /**
     * API will call getListOfHostMorefsFromDatastores API and will return the list of Esx host(s).
     * @param vcenterServerUuid : String
     * @param List<String> datastoresMorefs
     * @return List<Entity>
     * @throws ApiException
     */
    List<Entity> getListOfHostMorefsFromDataStores(NvpfVSphereSession session, List<String> datastoresMorefs) throws ApiException;
    
    
    
    /**
     * This API will attach a list of VMDKs to a VM from a backup.
     * @param session : NvpfVSphereSession
     * @param backupId : String
     * @param vmInstanceUuid : String
     * @param List<String> diskPathList
     * @return 
     * @throws ApiException
     */
    void attachVmdksToVM(NvpfVSphereSession session, String backupId, String vmInstanceUuid, List<String> diskPathList) throws ApiException;
    
    /** This method will fetch Detachable Vmdks for a VM
     * @param vmInstanceUuid
     * @return List<String>
     * @throws ApiException
     */
    List<String> getDetachableVmdksForVM(NvpfVSphereSession session, String vmInstanceUuid) throws ApiException;
    
    /** This method will Detach Vmdks from a VM and unmount the mounted datastore
     * @param vmInstanceUuid
     * @param diskPathList
     * @throws ApiException
     */
    void detachVmdksFromVM(NvpfVSphereSession session, String vmInstanceUuid, List<String> diskPathList) throws ApiException;

    /** This method will fetch Job Summary details of a backup job.
     * @param backupJobId
     * @param startTime
     * @param endTime
     * @return List<BackupJobSummary>
     * @throws ApiException
     */
    List<BackupJobSummary> getBackupJobSummary(String backupJobId, Date startTime, Date endTime) throws ApiException;

    /** This method will fetch virtual machine list for a particular datastore.
     * @param session
     * @param datastore
     * @return List<VirtualMachine>
     * @throws ApiException
     */
    List<VirtualMachine> getVirtualMachineList(NvpfVSphereSession session, Datastore datastore) throws ApiException;
    
    /**
     * @param session
     * @param globalConfiguration
     * @throws ApiException
     */
    void updateGlobalConfiguration(NvpfVSphereSession session, GlobalConfiguration globalConfiguration) throws ApiException;

}

