/*
 * $Id: //depot/prod/nvpf/Rinxs/src/java/com/netapp/nvpf/api/model/BackupRecoveryApiService.java#1 $
 *
 * Copyright (c) 2013 NetApp, Inc.
 * All rights reserved.
 * 
 */
package com.netapp.nvpf.api.model;

import java.util.Date;
import java.util.List;

import com.netapp.nvpf.api.model.backup.Backup;
import com.netapp.nvpf.api.model.backup.BackupJob;
import com.netapp.nvpf.api.model.backup.BackupJobSummary;
import com.netapp.nvpf.api.model.backup.BackupListResult;
import com.netapp.nvpf.api.model.backup.BackupLocations;
import com.netapp.nvpf.api.model.backup.BackupPolicy;
import com.netapp.nvpf.api.model.backup.BackupRequestFilter;
import com.netapp.nvpf.api.model.backup.Entity;
import com.netapp.nvpf.api.model.backup.ProtectionGroup;
import com.netapp.nvpf.api.model.backup.RestoreInfo;
import com.netapp.nvpf.api.model.backup.RestoreModel;
import com.netapp.nvpf.api.model.backup.configuration.EmailConfiguration;
import com.netapp.nvpf.api.model.backup.configuration.GlobalConfiguration;
import com.netapp.nvpf.api.model.backup.configuration.SnapManagerConfiguration;
import com.netapp.nvpf.api.model.datacenter.Datacenter;
import com.netapp.nvpf.api.model.datastore.Datastore;
import com.netapp.nvpf.api.model.virtualmachine.VirtualMachine;

/**
 * APIs related to back and recovery
 *
 */
public interface BackupRecoveryApiService {

    List<BackupJob> getBackupJobList(RequestQuery requestQuery, boolean includeAegisJobs, boolean includeSMVIJobs) throws ApiException;
    
    List<BackupJob> getBackupJobListByMoref(String moref, RequestQuery requestQuery) throws ApiException;

    BackupJob getBackupJobById(String backupJobId) throws ApiException;

    /**
     * API returns backupListResult for an entity. If the summaryOnly is false, backup objects
     * will also contain datastoreList and if true, datastoreList will be null.
     * @param moref
     * @param filter
     * @param summaryOnly
     * @param includeAegisBackups : boolean
     * @param includeSmviBackups : boolean
     * @return
     * @throws ApiException
     */
    BackupListResult getBackupList(String moref, BackupRequestFilter filter, boolean summaryOnly, boolean includeAegisBackups, boolean includeSmviBackups) throws ApiException;

    Backup getBackupById(String backupId) throws ApiException;

    void createBackup(String moref, Backup backup) throws ApiException;

    void deleteBackups(List<String> backupNames) throws ApiException;

    void renameBackup(String backupName, String newBackupName) throws ApiException;

    void deleteBackupJobs(List<String> jobs) throws ApiException;

    void mountBackup(String backupId, List<String> datastoreNames, String hostName, List<BackupLocations> backupLocators) throws ApiException;

    void updateBackupJob(BackupJob backupJob, boolean runJobNow, boolean dontSaveJob) throws ApiException;

    List<Datacenter> getDatacenterList() throws ApiException;

    List<VirtualMachine> getVirtualMachineList(Datastore datastore) throws ApiException;

    boolean isBackupJobNameExists(String backupJobName) throws ApiException;

    List<String> getAvailableScriptList() throws ApiException;

    void notificationTestMail(boolean nextGenBased, EmailConfiguration emailConfiguration) throws ApiException;

    List<Datastore> getSpanningDatastoreList(boolean nextGenBased, List<String> selectedEntities, boolean includeIndependentDisks) throws ApiException;

    void updateBackupJobs(List<BackupJob> backupJobs) throws ApiException;

    void runJobNow(BackupJob backupJob) throws ApiException;

    List<Datastore> getDatastoreListVM(String vmUuid) throws ApiException;

    void updateGlobalConfiguration(GlobalConfiguration globalConfiguration) throws ApiException;

    GlobalConfiguration getGlobalConfiguration() throws ApiException;

    void restore(RestoreModel restoreModel, List<BackupLocations> backupLocators) throws ApiException;

    void restoreVirtualDisk(RestoreModel restoreModel, List<BackupLocations> backupLocators) throws ApiException;

    RestoreInfo getRestoreInfo(String entityUuid, String entityName, String backupId) throws ApiException;

    List<Entity> getListOfEntityMorefsFromBackup(List<String> backupName) throws ApiException;

    List<Entity> getListOfHostMorefsFromBackups(List<String> backupName) throws ApiException;

    List<Entity> getMorefForEntities(List<Entity> selectedEntityList, boolean isSnapstarEntity) throws ApiException;

    int getBackupJobCount(String moref) throws ApiException;

    /**
     * API will call the getRegisteredHost to get SMS Server configuration details.
     * @return SnapManagerConfiguration details.
     * @throws ApiException
     */
    SnapManagerConfiguration getRegisteredHost() throws ApiException;

    /**
     * API will call the registerHost to register given SM configuration details to SMS Server
     * @param SnapManagerConfiguration
     * @throws ApiException
     */
    void registerHost(SnapManagerConfiguration snapManagerConfiguration) throws ApiException;

    /**
     * API will get the array of Policy Objects.
     * @param  requestQuery
     * @return BackupPolicy[]
     * @throws ApiException
     */
    BackupPolicy[] getPolicies(RequestQuery requestQuery) throws ApiException;

    /**
     * API will create a Protection Group on SnapManager Server.
     * @param protectionGroup
     * @throws ApiException
     */
    void createProtectionGroup(ProtectionGroup protectionGroup) throws ApiException;

    /**
     * Get the number of backups for a specific object
     * @param moref Moref of the object
     * @return Count of backups
     * @throws ApiException
     */
    int getBackupCount(String moref) throws ApiException;

    /**
     * Get the list of backups for a specific object with some filter/sort criteria
     * @param moref Moref of the object
     * @param requestQuery The query
     * @return List of backups
     * @throws ApiException
     */
    List<Backup> getBackupList(String moref, RequestQuery requestQuery) throws ApiException;

    /**
     * API will create the Backup by applying the given policy.
     * @param configName
     * @param policyName
     * @throws ApiException
     */
    void createBackupNow(String configName, String policyName) throws ApiException;

    /**
     * @param backupIds
     * @param datastoresUuid
     * @throws ApiException
     */
    void unmountBackups(String[] backupIds, String[] datastoresUuid) throws ApiException;

    /**
     * @param backupId
     * @return List of datastore mounted from the backup
     * @throws ApiException
     */
    List<Datastore> getMountedDatastores(String backupId) throws ApiException;

    /** 
     * API will update the protection group information on SnapManager Server.
     * @param protectionGroup
     * @throws ApiException
     */
    void updateProtectionGroup(ProtectionGroup protectionGroup) throws ApiException;

    /**
     * API will create the policy on SnapManager Server.
     * @param backupPolicy
     * @throws ApiException
     */
    void addPolicy(BackupPolicy backupPolicy) throws ApiException;

    /**
     * The method deletePolicy will delete policy using parameter policy id.
     * @param policyId
     * @throws ApiException
     */
    void deletePolicy(String policyId) throws ApiException;

    /**
     * The method getPolicyById will return policy using parameter policy id.
     * @param policyId
     * @throws ApiException
     */
    BackupPolicy getPolicyById(String policyId) throws ApiException;

    /**
     * This method will update policy.
     * @param backupPolicy
     * @throws ApiException
     */
    void updatePolicy(BackupPolicy backupPolicy) throws ApiException;

    /**
     * This method will fetch the list of filtered backups
     * @param session
     * @param moref
     * @param backupRequestFilter
     * @return
     * @throws ApiException
     */
    BackupListResult getBackupList(String moref, BackupRequestFilter backupRequestFilter) throws ApiException;

    /**
     * This method will check if the passed entities are Snap* compatible.
     * @param managedObjectReferenceList - List of Entities morefs.
     * @return boolean - true if compatible false otherwise
     * @throws ApiException
     */
    boolean isSnapStarCompatible(List<String> managedObjectReferenceList) throws ApiException;

    /**
     * API will get the list of Protection Groups by moref (property related to DS moref /VM uuid)
     * Set include true if you want to get list which contains entity with passed moref.
     * @param  moref
     * @param include
     * @param requestQuery
     * @return List<ProtectionGroup>
     * @throws ApiException
     */
    List<ProtectionGroup> getProtectionGroupsByMoref(String moref, boolean include, RequestQuery requestQuery) throws ApiException;

    /**
     * API will get the list of available scripts.
     * @param
     * @return List<String> of scripts.
     * @throws ApiException
     */
    List<String> getAvailableScriptsForPolicy() throws ApiException;

    /**
     * The method will get the policy using parameter policy name.
     * @param policyName String
     * @return BackupPolicy.
     * @throws ApiException
     */
    BackupPolicy getPolicyByName(String policyName) throws ApiException;

    /**
     * The method will get the BackupJob using parameter backupJobName.
     * @param backupJobName String
     * @return BackupJob.
     * @throws ApiException
     */
    BackupJob getBackupJobByName(String backupJobName) throws ApiException;
    
    /**
     * The method will get the ProtectionGroup using parameter backupJobName.
     * @param backupJobName String
     * @return ProtectionGroup.
     * @throws ApiException
     */
    ProtectionGroup getProtectionGroupByName(String backupJobName) throws ApiException;

    /**
     * This method will return Entity Object by using entityMoref
     * @param entityMoref : String
     * @return Entity
     * @throws ApiException
     */
    Entity getEntityByMoref(String entityMoref) throws ApiException;
    /**
     * This API will fetch datastore volume's primary and secondary locations for a specified backup.
     * @param backupId : String
     * @return List<BackupLocations>
     * @throws ApiException
     */
    List<BackupLocations> getDatastoreBackupLocators(String backupId) throws ApiException;

    /**
     * This API will fetch datastore volume's primary and secondary locations for a specified backup based on virtualEntityId and destDatastoreIds passed in restoreModel
     * @param backupId : String
     * @param restoreModel
     * @return List<BackupLocations>
     * @throws ApiException
     */
    List<BackupLocations> getDatastoreBackupLocatorsForRestore(String backupId, RestoreModel restoreModel) throws ApiException;

    /**
     * API will get list of Esx hosts for given datastoreMoref(s).
     * @param List<String> datastoresMorefs
     * @return List<Entity>
     * @throws ApiException
     */
    List<Entity> getListOfHostMorefsFromDataStores(List<String> datastoresMorefs) throws ApiException;
    
    /**
     * This API will attach a list of VMDKs to a VM from a backup.
     * @param backupId : String
     * @param vmInstanceUuid : String
     * @param List<String> diskPathList
     * @return 
     * @throws ApiException
     */
    void attachVmdksToVM(String backupId, String vmInstanceUuid, List<String> diskPathList) throws ApiException;
    
    /** This method will fetch Detachable Vmdks for a VM
     * @param vmInstanceUuid
     * @return List<String>
     * @throws ApiException
     */
    List<String> getDetachableVmdksForVM(String vmInstanceUuid) throws ApiException;
    
    /** This method will Detach Vmdks from a VM
     * @param vmInstanceUuid
     * @param diskPathList
     * @throws ApiException
     */
    void detachVmdksFromVM(String vmInstanceUuid, List<String> diskPathList) throws ApiException;

    /** This method will fetch Job Summary details of a backup job.
     * @param backupJobId
     * @param startTime
     * @param endTime
     * @return List<BackupJobSummary>
     * @throws ApiException
     */
    List<BackupJobSummary> getBackupJobSummary(String backupJobId, Date startTime, Date endTime) throws ApiException;
    
    /** This method will resume ProtectionGroups.
     * @param protectionGroupIds
     * @throws ApiException
     */
    void resumeProtectionGroups(List<String> protectionGroupIds) throws ApiException;
    
    /** This method will suspend ProtectionGroups.
     * @param protectionGroupIds
     * @throws ApiException
     */
    void suspendProtectionGroups(List<String> protectionGroupIds) throws ApiException;
}
