/*
 * $Id: //depot/prod/nvpf/Rinxs/src/java/com/netapp/nvpf/api/model/CasterApiService.java#1 $
 *
 * Copyright (c) 2013 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.nvpf.api.model;


import java.util.List;
import java.util.Map;

import com.netapp.nvpf.api.model.backup.WeeklySchedule;
import com.netapp.nvpf.api.model.datastore.Datastore;
import com.netapp.nvpf.api.model.datastore.DatastoreInitiateScanValidationResult;
import com.netapp.nvpf.api.model.datastore.DatastoreOptimizationResult;
import com.netapp.nvpf.api.model.virtualmachine.migration.MigrationModel;
import com.netapp.nvpf.api.model.virtualmachine.migration.ScannedVirtualMachine;
import com.netapp.nvpf.api.model.virtualmachine.migration.VirtualMachineAlignmentResult;

/**
 * Interface that will be implemented in the caster sub-plugin back-end
 * This interface declares the services available from caster. 
 *
 */
public interface CasterApiService {

    Map<String, Datastore> getDatastoresWithScannerDetails(List<String> morefs, NvpfVSphereSession session) throws ApiException;
    MigrationModel getMigrationModel(List<String> vmUuid, NvpfVSphereSession session) throws ApiException;
    /**
     * This method evaluates whether a specific storage controller is capable of optimization of datastore of a specific protocol type.
     *
     * @param provisioningControllerId ID of the storage controller
     * @param protocol Datastore protocol type - NFS or VMFS
     * @param session vSphere session ID
     * @throws ApiException Throws exception if the protocol is not supported for optimization
     * @return A null return value is a success, anything non-null is an error.
     */
    String validateControllerForMigration(String provisioningControllerId, String protocol, NvpfVSphereSession session) throws ApiException;
    List<Datastore> getDatastores(MigrationModel migrationModel, NvpfVSphereSession session) throws ApiException;
    void migrateVirtualMachines(MigrationModel migrationModel, NvpfVSphereSession session) throws ApiException;
    boolean checkDatastoreNameExistsByScannedVM(String datastoreName, ScannedVirtualMachine[] scannedVirtualMachine, NvpfVSphereSession session) throws ApiException;
    DatastoreOptimizationResult getDatastoreOptimizationDetails(int startIndex, int pageSize, String sortedColumn, boolean ascending, NvpfVSphereSession session) throws ApiException;
    VirtualMachineAlignmentResult getVirtualMachineAlignmentResult(NvpfVSphereSession session, int startIndex, int pageSize, 
    String sortedColumn, boolean ascending, String datastoreId, boolean misAlignedOnly) throws ApiException;
    DatastoreInitiateScanValidationResult startAlignmentScan(String[] datastoreMorefs, NvpfVSphereSession session) throws ApiException;
    String getScanJobStatus(String jobID) throws ApiException;
    void setDatastoresExcluded(String[] datastoresMoref, boolean exclude) throws ApiException;
    WeeklySchedule [] getScheduleTimes(NvpfVSphereSession session) throws ApiException;
    void saveScheduledTimes(List<WeeklySchedule> times, NvpfVSphereSession session)throws ApiException;
    DatastoreOptimizationResult getDatastoreOptimizationDetails(String sortedColumn, boolean ascending, NvpfVSphereSession session) throws ApiException;
    VirtualMachineAlignmentResult getVirtualMachineAlignmentResult(NvpfVSphereSession session, String sortedColumn, boolean ascending, boolean misAlignedOnly, String... datastoreId) throws ApiException;
    void migrateVirtualMachinesOnMultipleDatastore(MigrationModel migrationModel, NvpfVSphereSession session) throws ApiException;
}
