/*
 * $Id: //depot/prod/nvpf/Rinxs/src/java/com/netapp/nvpf/api/model/ExoforceApiService.java#1 $
 *
 * Copyright (c) 2012 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.nvpf.api.model;

import java.util.List;
import java.util.Map;
import java.util.Set;

import com.netapp.nvpf.api.model.datastore.Datastore;
import com.netapp.nvpf.api.model.host.Host;
import com.netapp.nvpf.api.model.host.HostDetails;
import com.netapp.nvpf.api.model.storagesystem.Controller;
import com.netapp.nvpf.api.model.storagesystem.ControllerPrivilegeOpinionBean;
import com.netapp.nvpf.api.model.storagesystem.ControllerTypeEnum;
import com.netapp.nvpf.api.model.storagesystem.LogicalInterface;
import com.netapp.nvpf.api.model.storagesystem.NasDetails;
import com.netapp.nvpf.api.model.storagesystem.NasDetailsHeader;
import com.netapp.nvpf.api.model.storagesystem.PathOptimizationDialogBean;
import com.netapp.nvpf.api.model.storagesystem.SanDetails;
import com.netapp.nvpf.api.model.storagesystem.SanDetailsHeader;
import com.netapp.nvpf.api.model.storagesystem.StorageSystemDiscoveryStatus;

/**
 * Interface that will be implemented in the exoforce sub-plugin back-end
 * This interface declares the services available from exoforce. 
 *
 */
public interface ExoforceApiService {

    List<Controller> getClusters(NvpfVSphereSession session) throws ApiException;

    List<Controller> getVServers(NvpfVSphereSession session) throws ApiException;

    List<Controller> getVFilers(NvpfVSphereSession session) throws ApiException;

    List<Controller> getSevenModeStorageSystems(NvpfVSphereSession session) throws ApiException;

    List<Controller> getUnknownStorageSystems(NvpfVSphereSession session) throws ApiException;

    List<Controller> getAllStorageSystems(NvpfVSphereSession session, RequestQuery requestQuery) throws ApiException;

    List<Controller> getAllStorageSystems(NvpfVSphereSession session, String moref, RequestQuery requestQuery) throws ApiException;

    int getControllerCount(NvpfVSphereSession session, String moref, ControllerTypeEnum type) throws ApiException;

    int getRelatedControllerCount(NvpfVSphereSession session, String controllerId, ControllerTypeEnum type) throws ApiException;

    int getRelatedInterfacesCount(NvpfVSphereSession session, String controllerId) throws ApiException;

    List<Controller> getVServers(String controllerId, NvpfVSphereSession session) throws ApiException;

    List<Controller> getVFilers(String controllerId, NvpfVSphereSession session) throws ApiException;

    ControllerPrivilegeOpinionBean getControllerPrivileges(Controller controller, Boolean isNewController, NvpfVSphereSession session) throws ApiException;

    Controller getControllerById(String controllerId, NvpfVSphereSession session) throws ApiException;

    List<LogicalInterface> getInterfaces(String controllerId, NvpfVSphereSession session, RequestQuery requestQuery) throws ApiException;

    String doDiscovery(NvpfVSphereSession session) throws ApiException;

    String doDiscovery(NvpfVSphereSession session, String moref) throws ApiException;

    Controller addController(Controller controller, NvpfVSphereSession session) throws ApiException;

    Controller updateController(Controller controller, NvpfVSphereSession session) throws ApiException;

    Boolean deleteController(Controller controller, NvpfVSphereSession session) throws ApiException;

    List<NasDetailsHeader> getNasDetailsHeaders(String controllerId, NvpfVSphereSession session, RequestQuery requestQuery) throws ApiException;

    List<SanDetailsHeader> getSanDetailsHeaders(String controllerId, NvpfVSphereSession session, RequestQuery requestQuery) throws ApiException;

    NasDetails getNasDetails(String datastore, String vmfspath, String remotehost,
            String remotefolder, NvpfVSphereSession session) throws ApiException;

    SanDetails getSanDetails(String serial, String durablename, NvpfVSphereSession session) throws ApiException;

    NasDetails getNasDetails(String datastoreMoref, NvpfVSphereSession session) throws ApiException;

    SanDetails getSanDetails(String datastoreMoref, NvpfVSphereSession session) throws ApiException;

    Map<String, Set<NasDetails>> getAllNasDetails(NvpfVSphereSession session) throws ApiException;

    Map<String, Set<SanDetails>> getAllSanDetails(NvpfVSphereSession session) throws ApiException;

    List<Host> getConnectedHosts(String controllerId, NvpfVSphereSession session) throws ApiException;

    List<Host> getHostList(NvpfVSphereSession session) throws ApiException;

    HostDetails getHostDetails(String moref, NvpfVSphereSession session) throws ApiException;

    Boolean skipHost(Host host, NvpfVSphereSession session) throws ApiException;

    Host updateHostNfsSettings(Host host, NvpfVSphereSession session) throws ApiException;

    Host updateHostHbaSettings(Host host, NvpfVSphereSession session) throws ApiException;

    Host updateHostMpioSettings(Host host, NvpfVSphereSession session) throws ApiException;

    PathOptimizationDialogBean getPathOptimizationDialogBean(String remotePath, String ip, NvpfVSphereSession session) throws ApiException;

    void setDefaultControllerCredentials(Controller controller) throws ApiException;

    Controller getDefaultControllerCredentials() throws ApiException;

    StorageSystemDiscoveryStatus getStorageSystemDiscoveryStatus(NvpfVSphereSession session) throws ApiException;

    String getNfsVaaiPluginVersion() throws ApiException;

    List<Host> getHostsForVaaiPluginInstall(NvpfVSphereSession session) throws ApiException;

    List<Host> getHostsForVaaiPluginInstall(String moref, NvpfVSphereSession session) throws ApiException;

    Boolean patchEsxHostsWithVaaiPlugin(NvpfVSphereSession session, List<Host> hosts, String pluginUrl) throws ApiException;

    List<Datastore> getDatastoreDetails(NvpfVSphereSession session, List<String> morefs) throws ApiException;

    /**
     * 
     * @param controllerId
     * @param session
     * @return
     * @throws ApiException
     */
    int getNasCount(String controllerId, NvpfVSphereSession session) throws ApiException;

    /**
     * 
     * @param controllerId
     * @param session
     * @return
     * @throws ApiException
     */
    int getSanCount(String controllerId, NvpfVSphereSession session) throws ApiException;
}
