/*
 * $Id: //depot/prod/nvpf/Rinxs/src/java/com/netapp/nvpf/api/model/HostApiService.java#1 $
 *
 * Copyright (c) 2012 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.nvpf.api.model;

import java.util.List;

import com.netapp.nvpf.api.model.host.Host;
import com.netapp.nvpf.api.model.host.HostDetails;
/**
 * APIs for ESX host related service 
 *
 */
public interface HostApiService { 

    /**
     * Gets a list of ESX hosts connected to a storage system
     * @param controllerId ID of teh controller
     * @return List of ESX hosts
     * @throws ApiException
     */
    List<Host> getConnectedHosts(String controllerId) throws ApiException;

    /**
     * Gets a list of all hosts discovered by VSC
     * @return List of hosts
     * @throws ApiException
     */
    List<Host> getHostList() throws ApiException;

    /**
     * Gets details of all settings for a specific ESX host
     * @param moref Moref for ESX host
     * @return Object containing the host details
     * @throws ApiException
     */
    HostDetails getHostDetails(String moref) throws ApiException;

    /**
     * Skips the monitoring of an ESX host
     * @param host The host object to be skipped
     * @return True if operation succeeded
     * @throws ApiException
     */
    Boolean skipHost(Host host) throws ApiException;

    /**
     * Updates the recommended NFS settings on an ESX host
     * @param host Host to be updated
     * @return Updated host
     * @throws ApiException
     */
    Host updateHostNfsSettings(Host host) throws ApiException;

    /**
     * Updates the recommended HBA settings on an ESX host
     * @param host Host to be updated
     * @return Updated host
     * @throws ApiException
     */
    Host updateHostHbaSettings(Host host) throws ApiException;

    /**
     * Updates the recommended MPIO settings on an ESX host
     * @param host Host to be updated
     * @return Updated host
     * @throws ApiException
     */
    Host updateHostMpioSettings(Host host) throws ApiException;

    /**
     * Get the Nfs VAAI plugin version.
     * @return Nfs VAAI plugin version.
     * @throws ApiException
     */
    String getNfsVaaiPluginVersion() throws ApiException;

    /**
     * Get List of Host for install Vaai Plugin.
     * @return
     * @throws ApiException
     */
    List<Host> getHostsForVaaiPluginInstall() throws ApiException;

    /**
     * Get List of Host for install Vaai Plugin.
     * @param moref
     * @return
     * @throws ApiException
     */
    List<Host> getHostsForVaaiPluginInstall(String moref) throws ApiException;

    /**
     * @param hosts
     * @return
     * @throws ApiException
     */
    Boolean patchEsxHostsWithVaaiPlugin(List<Host> hosts, String pluginUrl) throws ApiException;

    /**
     * 
     * @return
     * @throws ApiException
     */
    List<Host> getHostsForMountBackup() throws ApiException;

}
