/*
 * $Id: //depot/prod/nvpf/Rinxs/src/java/com/netapp/nvpf/api/model/KaminoApiService.java#1 $
 *
 * Copyright (c) 2012 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.nvpf.api.model;


import java.util.List;
import java.util.Map;
import java.util.Set;

import com.netapp.nvpf.api.model.cb.ConnectionBroker;
import com.netapp.nvpf.api.model.datastore.Datastore;
import com.netapp.nvpf.api.model.datastore.ResizeDatastoreInfo;
import com.netapp.nvpf.api.model.datastore.provisioning.DatastoreCapability;
import com.netapp.nvpf.api.model.datastore.provisioning.ProvisioningDatastoreSpec;
import com.netapp.nvpf.api.model.datastore.provisioning.ProvisioningMessage;
import com.netapp.nvpf.api.model.datastore.provisioning.ProvisioningTargetController;
import com.netapp.nvpf.api.model.datastore.provisioning.ProvisioningTargetControllerDetail;
import com.netapp.nvpf.api.model.storagesystem.Controller;
import com.netapp.nvpf.api.model.storagesystem.DedupeData;
import com.netapp.nvpf.api.model.storagesystem.DedupeItem;
import com.netapp.nvpf.api.model.storagesystem.QosThroughput;
import com.netapp.nvpf.api.model.virtualmachine.VirtualMachine;
import com.netapp.nvpf.api.model.virtualmachine.VirtualMachineFile;
import com.netapp.nvpf.api.model.virtualmachine.cloning.CloningLimits;
import com.netapp.nvpf.api.model.virtualmachine.cloning.CloningModel;
import com.netapp.nvpf.api.model.virtualmachine.cloning.InventoryItem;
import com.netapp.nvpf.api.model.virtualmachine.cloning.RedeployClones;

/**
 * Interface that will be implemented in the kamino sub-plugin back-end
 * This interface declares the services available from kamino. 
 *
 */
public interface KaminoApiService {

    Controller getControllerProvisioningDetails(String controllerId, boolean refreshController,
        NvpfVSphereSession session) throws ApiException;

    void confirmCredentials(Controller controller, NvpfVSphereSession session)
        throws ApiException;

    DedupeItem getStroageCapacityDetail(String controllerId, NvpfVSphereSession session)
        throws ApiException;

    void updateDedupeOnDataStore(String moref, DedupeData dedupeData, NvpfVSphereSession session)
        throws ApiException;

    void updateProvisioningOptions(Controller controllerProvisioningSettings,
        NvpfVSphereSession session) throws ApiException;

    void updateProvisioningResources(Controller controllerProvisioningResources,
        NvpfVSphereSession session) throws ApiException;

    Boolean checkDatastoreNameExists(String datastoreName, String moref, NvpfVSphereSession session) throws ApiException;
    
    List<ProvisioningTargetController> getProvisionableControllers(String datastoreType, String protocol) throws ApiException;
    
    List<ProvisioningTargetController> getProvisionableControllersForPolicy(String datastoreType, String protocol, String policy) throws ApiException;
    
    Map<String, List<ProvisioningMessage>> getLicensingErrors(NvpfVSphereSession session, String moref) throws ApiException;
    
    ProvisioningTargetControllerDetail getProvisioningTargetControllerDetail(String controllerId, String parentControllerId,
            String moref, String datastoreType, String protocol, NvpfVSphereSession session) throws ApiException;
    
    ProvisioningTargetControllerDetail getProvisioningTargetControllerDetailForPolicy(String controllerId, String parentControllerId,
            String moref, String datastoreType, String protocol, String policy, NvpfVSphereSession session) throws ApiException;

    Boolean createDatastore(String controllerId, ProvisioningDatastoreSpec pds, NvpfVSphereSession session) throws ApiException;

    Datastore getDatastoreDestroyInfo(String moref, NvpfVSphereSession session) throws ApiException;
    
    List<Datastore> getMountableDatastores(String moref, NvpfVSphereSession session) throws ApiException;

    void mountDatastores(String moref, List<Datastore> datastores, NvpfVSphereSession session) throws ApiException;

    void destroyDatastore(String moref, String controllerId, NvpfVSphereSession session) throws ApiException;
    
    Datastore getDatastoreReclaimSpaceInfo(String moref, NvpfVSphereSession session) throws ApiException;

    void reclaimSpaceForDatastore(String moref, String controllerId, NvpfVSphereSession session) throws ApiException;
    
    ResizeDatastoreInfo getDatastoreResizeInfo(String moref, NvpfVSphereSession session) throws ApiException;

    void resizeDatastoreForVMFS(String moref, String controllerId, double newSize, NvpfVSphereSession session) throws ApiException;

    void resizeDatastoreForNFS(String moref, String controllerId, double newSize, String containerName, NvpfVSphereSession session) throws ApiException;

    VirtualMachine getVirtualMachineInfo(String moref, NvpfVSphereSession session) throws ApiException;
    
    VirtualMachine getVirtualMachineByMoref(String moref, NvpfVSphereSession session) throws ApiException;
    
    List<VirtualMachine> getVirtualMachineListByMoref(List<String> morefList, NvpfVSphereSession session) throws ApiException;

    void reclaimSpace(String moref, NvpfVSphereSession session) throws ApiException;
    
    Boolean isProvisioning(); 
    
    List<ConnectionBroker> getConnectionBrokers() throws ApiException;
    
    
    /**
     * Return cloning limits for virtualmachine
     * @param session
     * @param inputCloningLimits
     * @return
     */
    CloningLimits getCloningLimits(NvpfVSphereSession session, CloningLimits inputCloningLimits);
    
    /**
     * Return Cloning Model with virtual machine object on which the cloning
     * wizard has been launched.
     * @param session
     * @param moref
     * @return
     */
    CloningModel getCloningModel(NvpfVSphereSession session, String moref) throws ApiException;
    
    Map<String, List<ProvisioningMessage>> getLicensingErrors(NvpfVSphereSession session, InventoryItem destination) throws ApiException;
    
    Boolean removeConnectionBrokers(List<ConnectionBroker> connectionBrokersToRemove) throws ApiException;
    
    InventoryItem getDestinationHostDetail(NvpfVSphereSession session, String moref) throws ApiException;
    
    Boolean updateConnectionBroker(ConnectionBroker connectionBroker) throws ApiException;
    
    InventoryItem getVmFolderTreeModel(NvpfVSphereSession session, String moref) throws ApiException;
    
    List<ProvisioningTargetController> getControllersForVMCloning(NvpfVSphereSession session, String moref) 
        throws ApiException;
    
    /**
     * The API will fetch usableDatastores based on controllerId and selected Inventory Item. 
     * Transform the kamnioDatastores to nvpfDatastores and return the datastoresForVMCloning.
     * @param session
     * @param moref
     * @param controllerId
     * @param rootInventoryItem
     * @param selectedInventoryItem
     * @return
     * @throws ApiException
     */
    List<Datastore> getDatastoresForVMCloning(NvpfVSphereSession session, String moref, String controllerId, 
        InventoryItem rootInventoryItem, InventoryItem selectedInventoryItem) throws ApiException;
    
    void createClones(NvpfVSphereSession session, String moref, CloningModel cloningModel) throws ApiException;
    
    boolean checkDatastoreSpace(NvpfVSphereSession session, String moref, CloningModel cloningModel) throws ApiException;
    
    List<String> getExistingViewPoolNames(NvpfVSphereSession session, String moref, String viewServerNameOrIp, String domain, String username, String password,
        boolean manualPoolsOnly) throws ApiException;
    /**
     * The API will fetch virtualMachine files based on controllerId and selected Inventory Item. 
     * @param session
     * @param moref
     * @param controllerId
     * @param rootInventoryItem
     * @param selectedInventoryItem
     * @return
     * @throws ApiException
     */
    List<VirtualMachineFile> getVirtualMachineFilesForVMCloning(NvpfVSphereSession session, String moref, String controllerId, 
        InventoryItem rootInventoryItem, InventoryItem selectedInventoryItem) throws ApiException;
    
    List<VirtualMachine> getVirtualMachinesFromBaseline(NvpfVSphereSession session, String moref, 
        String instanceUuid, String uuid) throws ApiException;
    
    void redeployClones(NvpfVSphereSession session, RedeployClones redeployClones) throws ApiException;

    Set<VirtualMachine> getVirtualMachines(NvpfVSphereSession session) throws ApiException;
    
    /**
     * API will set existingDatastoreNames and existingVolumeNames in CloningLimits
     * @param inventoryItem
     * @param controllerID
     * @return
     * @throws ApiException
     */
    CloningLimits getCloningLimitsForNFSDatastoreConfiguration(InventoryItem inventoryItem, 
            String controllerID, String moref, NvpfVSphereSession session) throws ApiException;
    
    Boolean validateDatastoreNameAgainstVolumeAndLunNames(String datastoreName, String controllerID, String datastoreType, String volumeName, String moref,
        NvpfVSphereSession session) throws ApiException;
    
    void setVMQos(NvpfVSphereSession session, String moref, QosThroughput qosSetting) throws ApiException;

    List<String> getPolicies() throws ApiException;
    
    Boolean isPolicyBasedProvisioningSupported() throws ApiException;
    
    List<DatastoreCapability> getDatastoreCapabilitiesForProfile(final String profileName) throws ApiException;

    boolean doesHardwareVersionUpgradeApply(VirtualMachine source, InventoryItem destination);
}
