/*
 * $Id: //depot/prod/nvpf/Rinxs/src/java/com/netapp/nvpf/api/model/ModelObject.java#1 $
 *
 * Copyright (c) 2012 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.nvpf.api.model;

import java.beans.PropertyDescriptor;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLEncoder;
import java.util.List;

/**
 * A super class for all the model beans used by the VSC APIs
 * This class abstracts some of the details required for the 
 * VSC flex client like the object URI.
 * 
 */
public abstract class ModelObject implements Serializable {

    private static final long serialVersionUID = 1L;
    public static final String SCHEME = "vsc";
    public static final String NAMESPACE = "vsc";
    public static final char URI_SEPARATOR = ':';
    public static final char FRAGMENT_SEPARATOR = '/';
    public static final String UID_PREFIX = SCHEME + URI_SEPARATOR + NAMESPACE;

    private String id;
    private String vcenterServerUuid;

    public URI getURI() throws URISyntaxException, UnsupportedEncodingException {
        String formattedId = id;
        if (id != null) {
            formattedId = URLEncoder.encode(id, "UTF-8");
        }
        return new URI(getFormattedURIString(formattedId));
    }
    
    protected String getFormattedURIString(String formattedId) {
        return String.format("%s:%s:%s:%s/%s", SCHEME, NAMESPACE, getTypeStr(), vcenterServerUuid, formattedId);
    }

    public String getId() {
        return this.id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getVcenterServerUuid() {
        return vcenterServerUuid;
    }

    public void setVcenterServerUuid(String vcenterServerUuid) {
        this.vcenterServerUuid = vcenterServerUuid;
    }

    /**
     * This method returns the model object's Class name e.g. Interface.
     * Overridden version of this method in Controller.java returns the specific
     * Controller type e.g. Cluster, VServer etc.
     */
    public String getTypeStr() {
        return this.getClass().getSimpleName();
    }

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder();
        builder.append("id=");
        builder.append(id);
        builder.append(", type=");
        builder.append(getTypeStr());
        try {
            builder.append(", URI=");
            builder.append(getURI());
        }
        catch (URISyntaxException e) {
            // Ignore this exception
        }
        catch (UnsupportedEncodingException e) {
         // Ignore this exception
        }

        return builder.toString();
    }

    public String formatAsString(List<String> listOfString) {
        StringBuffer sbOut = new StringBuffer();
        if (listOfString != null) {
            int size = listOfString.size();
            for (int i = 0; i < size; i++) {
                sbOut.append(listOfString.get(i));
                if (i < size - 1) {
                    sbOut.append(", ");
                }
            }
        }
        return sbOut.toString();
    }
    
    /**
     * Helper method for flex web client to extract the value of a property by name
     * @param propertyName
     * @return
     */
    public Object getProperty(String propertyName) {
        Object propertyValue = null;
        try {
            propertyValue = new PropertyDescriptor(propertyName, 
                this.getClass()).getReadMethod().invoke(this);
            if (propertyValue != null && propertyValue instanceof Enum) {
                propertyValue = propertyValue.toString();
            }
        }
        catch (Exception e) {
            e.printStackTrace();
            //We will return null, if the property could not be read or if it did not have
            //a proper access method
        }
        return propertyValue;
    }
}
