/*
 * $Id: //depot/prod/nvpf/Rinxs/src/java/com/netapp/nvpf/api/model/SmviApiService.java#1 $
 * 
 * Copyright (c) 2013 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.nvpf.api.model;

import java.util.List;

import com.netapp.nvpf.api.model.backup.Backup;
import com.netapp.nvpf.api.model.backup.BackupJob;
import com.netapp.nvpf.api.model.backup.BackupListResult;
import com.netapp.nvpf.api.model.backup.BackupRequestFilter;
import com.netapp.nvpf.api.model.backup.Entity;
import com.netapp.nvpf.api.model.backup.RestoreInfo;
import com.netapp.nvpf.api.model.backup.RestoreModel;
import com.netapp.nvpf.api.model.backup.configuration.EmailConfiguration;
import com.netapp.nvpf.api.model.backup.configuration.GlobalConfiguration;
import com.netapp.nvpf.api.model.datacenter.Datacenter;
import com.netapp.nvpf.api.model.datastore.Datastore;
import com.netapp.nvpf.api.model.host.Host;
import com.netapp.nvpf.api.model.virtualmachine.VirtualMachine;

/**
 * APIs exposed by the SMVI sub-plugin
 *
 */
public interface SmviApiService {

    List<BackupJob> getBackupJobList(NvpfVSphereSession session, RequestQuery requestQuery) throws ApiException;
    List<BackupJob> getBackupJobListByMoref(NvpfVSphereSession session, String moref, RequestQuery requestQuery) throws ApiException;
    BackupJob getBackupJobById(NvpfVSphereSession session, String backupJobId) throws ApiException;
    List<Backup> getBackupList(NvpfVSphereSession session, String moref, RequestQuery requestQuery) throws ApiException;
    BackupListResult getBackupList(NvpfVSphereSession session, String moref, BackupRequestFilter filter, boolean summaryOnly) throws ApiException;
    Backup getBackupById(NvpfVSphereSession session, String backupId) throws ApiException;
    void createBackup(NvpfVSphereSession session, String moref, Backup backup) throws ApiException;
    void deleteBackups(NvpfVSphereSession session, List<String> backupNames) throws ApiException;
    void renameBackup(NvpfVSphereSession session, String backupName, String newbackupName) throws ApiException;
    //deleteBackupJobs is to delete selected backup jobs from UI, where jobs are List of Jobnames.
    void deleteBackupJobs(NvpfVSphereSession session, List<String> jobs) throws ApiException;
    List<Host> getHostsForMountBackup(NvpfVSphereSession session) throws ApiException;
    void mountBackup(NvpfVSphereSession session, String backupName, String hostName) throws ApiException;
    void updateBackupJob(NvpfVSphereSession session, BackupJob backupJob, boolean runJobNow, boolean dontSaveJob) throws ApiException;
    List<Datacenter> getDatacenterList(NvpfVSphereSession session) throws ApiException;
    List<VirtualMachine> getVirtualMachineList(NvpfVSphereSession session, Datastore datastore) throws ApiException;
    boolean isBackupJobNameExists(NvpfVSphereSession session, String backupJobName) throws ApiException;
    List<String> getAvailableScriptList(NvpfVSphereSession session) throws ApiException;
    void notificationTestMail(NvpfVSphereSession session, EmailConfiguration emailConfiguration) throws ApiException;
    List<Datastore> getSpanningDatastoreList(NvpfVSphereSession session, List<String> selectedEntities, boolean includeIndependentDisks) throws ApiException;
    void updateBackupJobs(NvpfVSphereSession session, List<BackupJob> backupJobs) throws ApiException;
    void unmountBackup(NvpfVSphereSession session, List<String> backupNames) throws ApiException;
    void runJobNow(NvpfVSphereSession session, BackupJob backupJob) throws ApiException;
    List<Datastore> getDatastoreListVM(NvpfVSphereSession session, String vmUuid) throws ApiException;
    void updateGlobalConfiguration(NvpfVSphereSession session, GlobalConfiguration globalConfiguration) throws ApiException;
    GlobalConfiguration getGlobalConfiguration(NvpfVSphereSession session) throws ApiException;
    void restore(NvpfVSphereSession session, RestoreModel restoreModel) throws ApiException;
    void restoreVirtualDisk(NvpfVSphereSession session, RestoreModel restoreModel) throws ApiException;
    RestoreInfo getRestoreInfo(NvpfVSphereSession session, String entityUuid, String entityName, String backupUuid) throws ApiException;
    List<Entity> getListOfEntityMorefsFromBackup(NvpfVSphereSession session, List<String> backupName) throws ApiException;
    List<Entity> getListOfHostMorefsFromBackups(NvpfVSphereSession session, List<String> backupName) throws ApiException;
    List<Entity> getMorefForEntities(NvpfVSphereSession session, List<Entity> selectedEntityList) throws ApiException;
    int getBackupJobCount(NvpfVSphereSession session, String moref) throws ApiException;
    int getBackupCount(NvpfVSphereSession session, String moref) throws ApiException;
}
