/*
 * $Id: $
 * 
 * Copyright (c) 2013 NetApp, Inc.
 * All rights reserved.
 * 
 */
package com.netapp.nvpf.api.model;

import java.util.List;
import java.util.Map;
import java.util.Set;

import com.netapp.nvpf.api.model.backup.WeeklySchedule;
import com.netapp.nvpf.api.model.datastore.Datastore;
import com.netapp.nvpf.api.model.datastore.provisioning.ProvisioningMessage;
import com.netapp.nvpf.api.model.datastore.provisioning.ProvisioningTargetController;
import com.netapp.nvpf.api.model.storagesystem.QosThroughput;
import com.netapp.nvpf.api.model.virtualmachine.VirtualMachine;
import com.netapp.nvpf.api.model.virtualmachine.VirtualMachineFile;
import com.netapp.nvpf.api.model.virtualmachine.cloning.CloningLimits;
import com.netapp.nvpf.api.model.virtualmachine.cloning.CloningModel;
import com.netapp.nvpf.api.model.virtualmachine.cloning.InventoryItem;
import com.netapp.nvpf.api.model.virtualmachine.cloning.RedeployClones;
import com.netapp.nvpf.api.model.virtualmachine.migration.MigrationModel;
import com.netapp.nvpf.api.model.virtualmachine.migration.ScannedVirtualMachine;
import com.netapp.nvpf.api.model.virtualmachine.migration.VirtualMachineAlignmentResult;


/**
 * @author chandraprakash.joshi
 *
 */
public interface VirtualMachineApiService {
    
    VirtualMachine getVirtualMachineInfo(String moref) throws ApiException;
    
    VirtualMachine getVirtualMachineByMoref(String moref) throws ApiException;
    
    List<VirtualMachine> getVirtualMachineListByMoref(List<String> morefList) throws ApiException;

    void reclaimSpace(String moref) throws ApiException;
    
    /**
     * Return cloning limits for virtualmachine
     * @param inputCloningLimits
     * @return
     */
    CloningLimits getCloningLimits(CloningLimits inputCloningLimits);
    CloningModel getCloningModel(String moref) throws ApiException;
    
    /**
     * Fetch the all licensing errors from the request host/datacenter moref field value
     * @param moref
     * @return
     * @throws ApiException
     */
    Map<String, List<ProvisioningMessage>> getLicensingErrors(InventoryItem destination) throws ApiException;
    
    /**
     * Return Destination host Detail
     * @return
     * @throws ApiException
     */
    InventoryItem getDestinationHostDetail(String moref) throws ApiException;
    
    InventoryItem getVmFolderTreeModel(String moref) throws ApiException;
    
    List<ProvisioningTargetController> getControllersForVMCloning(String moref) throws ApiException;
    
    List<Datastore> getDatastoresForVMCloning(String moref, String controllerId, InventoryItem rootInventoryItem, InventoryItem selectedInventoryItem) throws ApiException;
    
    void createClones(String moref, CloningModel cloningModel) throws ApiException;
    
    boolean checkDatastoreSpace(String moref, CloningModel cloningModel) throws ApiException;
    
    List<String> getExistingViewPoolNames(String moref, String viewServerNameOrIp, String domain, String username, String password, boolean manualPoolsOnly) throws ApiException;
    List<VirtualMachineFile> getVirtualMachineFilesForVMCloning(String moref, String controllerId, InventoryItem rootInventoryItem, InventoryItem selectedInventoryItem) throws ApiException;
    
    List<VirtualMachine> getVirtualMachinesFromBaseline(String moref, String instanceUuid, String uuid) throws ApiException;
    
    void redeployClones(RedeployClones redeployClones) throws ApiException;
    Set<VirtualMachine> getVirtualMachines() throws ApiException;
    
    MigrationModel getMigrationModel(List<String> vmUuid) throws ApiException;
    /**
     *This method evaluates whether a specific storage controller is capable of optimization of datastore of a specific protocol type.
     *
     * @param provisioningControllerId ID of the storage controller.
     * @param protocol Datastore protocol type - NFS or VMFS.
     * @param session vSphere session ID.
     * @throws ApiException Throws exception if the protocol is not supported for optimization.
     * @return A null return value is a success, anything non-null is an error.
     */
    String validateControllerForMigration(String provisioningControllerId, String protocol) throws ApiException;
    CloningLimits getCloningLimitsForNFSDatastoreConfiguration(InventoryItem inventoryItem, String controllerID, String moref) throws ApiException;
    void migrateVirtualMachines(MigrationModel migrationModel) throws ApiException;
    boolean checkDatastoreNameExistsByScannedVM(String datastoreName, ScannedVirtualMachine[] scannedVirtualMachine) throws ApiException;
    VirtualMachineAlignmentResult getVirtualMachineAlignmentResult(int startIndex, int pageSize, 
        String sortedColumn, boolean ascending, String datastoreId, boolean misAlignedOnly) throws ApiException;
    void setDatastoresExcluded(String[] datastoresMoref, boolean exclude) throws ApiException;
    
    WeeklySchedule [] getScheduleTimes() throws ApiException;
    
    void saveScheduledTimes(List<WeeklySchedule> weeklySchedules)throws ApiException;
    
    void setVMQos(String moref, QosThroughput qosThroughput) throws ApiException;

    boolean doesHardwareVersionUpgradeApply(VirtualMachine source, InventoryItem destination);
}
