/*
 * $Id: $
 * 
 * Copyright (c) 2013 NetApp, Inc.
 * All rights reserved.
 * 
 */
package com.netapp.nvpf.api.model.backup;

import java.util.Date;

import com.netapp.nvpf.api.model.ModelObject;
import com.netapp.nvpf.api.model.datastore.Datastore;

public class Backup extends ModelObject implements Comparable<Backup> {

    private static final long serialVersionUID = -7299262945363902912L;
    private static final int SFR_SUPPORTED_BACKUP_VERSION = 2;


    private String name;
    private String uuid;

    private Date startTime;
    private Date endTime;
    private Boolean mounted;
    private Boolean mountedForSFR;
    private int smviVersion;
    private boolean noVmSnaps;
    private BackupMount manualBackupMount;
    private BackupMount sfsrBackupMount;
    private boolean updateSnapMirror;
    private boolean updateSnapVault;
    private boolean includeIndependentDisks;
    private EntitySummary[] entitySummaries;
    private Datastore[] datastores;
    private String vCenterUuid;
    // indicates whether it's an SMVI backup job or a Next-Gen backup job.
    // 0 - SMVI BackupJob and 1 - NextGen BackupJob
    private int version;

    /**
     * @return the mountedForSFR
     */
    public Boolean getMountedForSFR() {
        return mountedForSFR;
    }
    /**
     * @param mountedForSFR the mountedForSFR to set
     */
    public void setMountedForSFR(Boolean mountedForSFR) {
        this.mountedForSFR = mountedForSFR;
    }
    /**
     * @return the manualBackupMount
     */
    public BackupMount getManualBackupMount() {
        return manualBackupMount;
    }
    /**
     * @param manualBackupMount the manualBackupMount to set
     */
    public void setManualBackupMount(BackupMount manualBackupMount) {
        this.manualBackupMount = manualBackupMount;
    }
    /**
     * @return the sfsrBackupMount
     */
    public BackupMount getSfsrBackupMount() {
        return sfsrBackupMount;
    }
    /**
     * @param sfsrBackupMount the sfsrBackupMount to set
     */
    public void setSfsrBackupMount(BackupMount sfsrBackupMount) {
        this.sfsrBackupMount = sfsrBackupMount;
    }



    /**
     * @return the name
     */
    public String getName() {
        return name;
    }
    /**
     * @param name the name to set
     */
    public void setName(String name) {
        this.name = name;
    }
    /**
     * @return the startTime
     */
    public Date getStartTime() {
        return startTime;
    }
    /**
     * @param startTime the startTime to set
     */
    public void setStartTime(Date startTime) {
        this.startTime = startTime;
    }
    /**
     * @return the endTime
     */
    public Date getEndTime() {
        return endTime;
    }
    /**
     * @param endTime the endTime to set
     */
    public void setEndTime(Date endTime) {
        this.endTime = endTime;
    }
    /**
     * @return the mounted
     */
    public Boolean getMounted() {
        return mounted;
    }
    /**
     * @param mounted the mounted to set
     */
    public void setMounted(Boolean mounted) {
        this.mounted = mounted;
    }
    /**
     * @return the smviVersion
     */
    public int getSmviVersion() {
        return smviVersion;
    }
    /**
     * @param smviVersion the smviVersion to set
     */
    public void setSmviVersion(int smviVersion) {
        this.smviVersion = smviVersion;
    }
    /**
     * @return the noVmSnaps
     */
    public boolean isNoVmSnaps() {
        return noVmSnaps;
    }
    /**
     * @param noVmSnaps the noVmSnaps to set
     */
    public void setNoVmSnaps(boolean noVmSnaps) {
        this.noVmSnaps = noVmSnaps;
    }


    public boolean isSFRSupportedVersion() {
        if (getSmviVersion() < SFR_SUPPORTED_BACKUP_VERSION) {
            return false;
        }
        return true;
    }
    /**
     * @return the updateSnapMirror
     */
    public boolean isUpdateSnapMirror() {
        return updateSnapMirror;
    }
    /**
     * @param updateSnapMirror the updateSnapMirror to set
     */
    public void setUpdateSnapMirror(boolean updateSnapMirror) {
        this.updateSnapMirror = updateSnapMirror;
    }
    /**
     * @return the includeIndependentDisks
     */
    public boolean isIncludeIndependentDisks() {
        return includeIndependentDisks;
    }
    /**
     * @param includeIndependentDisks the includeIndependentDisks to set
     */
    public void setIncludeIndependentDisks(boolean includeIndependentDisks) {
        this.includeIndependentDisks = includeIndependentDisks;
    }

    @Override
    public int compareTo(Backup backupObj) {
        if (getStartTime() == null || backupObj.getStartTime() == null) {
            return 0;
        }
        else {
            return backupObj.getStartTime().compareTo(getStartTime());
        }
    }
    public Datastore[] getDatastores() {
        return datastores;
    }
    public void setDatastores(Datastore[] datastores) {
        this.datastores = datastores;
    }
    public String getUuid() {
        return uuid;
    }
    public void setUuid(String uuid) {
        this.uuid = uuid;
    }
    public EntitySummary[] getEntitySummaries() {
        return entitySummaries;
    }
    public void setEntitySummaries(EntitySummary[] entitySummaries) {
        this.entitySummaries = entitySummaries;
    }
    public boolean isUpdateSnapVault() {
        return updateSnapVault;
    }
    public void setUpdateSnapVault(boolean updateSnapVault) {
        this.updateSnapVault = updateSnapVault;
    }

    public String getvCenterUuid() {
        return vCenterUuid;
    }

    public void setvCenterUuid(String vCenterUuid) {
        this.vCenterUuid = vCenterUuid;
    }

    public int getVersion() {
        return version;
    }
    public void setVersion(int version) {
        this.version = version;
    }

}
