/*
 * $Id: //depot/prod/nvpf/Rinxs/src/java/com/netapp/nvpf/api/model/storagesystem/Controller.java#1 $
 *
 * Copyright (c) 2012 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.nvpf.api.model.storagesystem;

import java.util.List;
import java.util.Map;
import java.util.Set;

import com.netapp.nvpf.api.model.ModelObject;

public class Controller extends ModelObject {

    private static final long serialVersionUID = -5945348445022511774L;

    private String name;
    private String model;
    private String ipAddress;
    private String partner;
    private Integer port;
    private String username;
    private String password;
    private boolean useSSL;
    private boolean multistore;
    private Integer overallStatus;
    private String overallStatusReason;
    private String version;
    private String cfMode;
    private String cfStatus;
    private Long totalCapacity;
    private Long totalAllocated;
    private Long totalUsed;
    private Long totalFree;
    private Set<String> interfaces;
    private String vaaiCapable;
    private byte supportedProtocols;
    private ControllerTypeEnum type;
    private Integer flagSet;
    private boolean clusteredOntap;
    private boolean updating;
    private boolean clusterNode;
    private boolean clusterVserver;
    private boolean clusterAdmin;

    private String controllerGroupingName;
    private Set<String> ipAddressList;
    private Map<String, Volume> volumes;
    private Map<String, Lun> luns;
    private List<Aggregate> aggregate;
    private List<Adapter> adapter;
    private Map<String, Qtree> qtree;

    private Set<String> availableInterfaceNames;
    private Set<String> usableInterfaceNames;

    private Set<String> availableVolumeNames;
    private Set<String> usableVolumeNames;

    private Set<String> availableAggregateNames;
    private Set<String> usableAggregateNames;

    private boolean isCreateVolumeWrapper;
    private boolean isSpaceReserveNewVolumes;
    private boolean isThinProvisionVolumeClones;
    private boolean isDeleteVolumeIfLastLun;
    private int volumeWrapperBuffer;
    

    public Set<String> getAvailableInterfaceNames() {
        return availableInterfaceNames;
    }

    public void setAvailableInterfaceNames(Set<String> availableInterfaceNames) {
        this.availableInterfaceNames = availableInterfaceNames;
    }

    public Set<String> getUsableInterfaceNames() {
        return usableInterfaceNames;
    }

    public void setUsableInterfaceNames(Set<String> usableInterfaceNames) {
        this.usableInterfaceNames = usableInterfaceNames;
    }

    public Set<String> getAvailableVolumeNames() {
        return availableVolumeNames;
    }

    public void setAvailableVolumeNames(Set<String> availableVolumeNames) {
        this.availableVolumeNames = availableVolumeNames;
    }

    public Set<String> getUsableVolumeNames() {
        return usableVolumeNames;
    }

    public void setUsableVolumeNames(Set<String> usableVolumeNames) {
        this.usableVolumeNames = usableVolumeNames;
    }

    public Set<String> getAvailableAggregateNames() {
        return availableAggregateNames;
    }

    public void setAvailableAggregateNames(Set<String> availableAggregateNames) {
        this.availableAggregateNames = availableAggregateNames;
    }

    public Set<String> getUsableAggregateNames() {
        return usableAggregateNames;
    }

    public void setUsableAggregateNames(Set<String> usableAggregateNames) {
        this.usableAggregateNames = usableAggregateNames;
    }

    public Set<String> getIpAddressList() {
        return ipAddressList;
    }

    public void setIpAddressList(Set<String> ipAddressList) {
        this.ipAddressList = ipAddressList;
    }

    public boolean isClusterNode() {
        return clusterNode;
    }

    public void setClusterNode(boolean clusterNode) {
        this.clusterNode = clusterNode;
    }

    public boolean isClusterVserver() {
        return clusterVserver;
    }

    public void setClusterVserver(boolean clusterVserver) {
        this.clusterVserver = clusterVserver;
    }

    public boolean isClusterAdmin() {
        return clusterAdmin;
    }

    public void setClusterAdmin(boolean clusterAdmin) {
        this.clusterAdmin = clusterAdmin;
    }

    public Integer getFlagSet() {
        return flagSet;
    }

    public void setFlagSet(Integer flagSet) {
        this.flagSet = flagSet;
    }

    public boolean isClusteredOntap() {
        return clusteredOntap;
    }

    public void setClusteredOntap(boolean clusteredOntap) {
        this.clusteredOntap = clusteredOntap;
    }

    public boolean isUpdating() {
        return updating;
    }

    public void setUpdating(boolean updating) {
        this.updating = updating;
    }

    public boolean isUseSSL() {
        return useSSL;
    }

    public void setUseSSL(boolean useSSL) {
        this.useSSL = useSSL;
    }

    public ControllerTypeEnum getType() {
        return type;
    }

    @Override
    public String getTypeStr() {
        String strType = null;
        if (getType() != null) {
            strType = getType().toString();
        }
        return strType;
    }

    public void setType(ControllerTypeEnum type) {
        this.type = type;
    }

    public String getVaaiCapable() {
        return vaaiCapable;
    }

    public void setVaaiCapable(String vaaiCapable) {
        this.vaaiCapable = vaaiCapable;
    }

    public byte getSupportedProtocols() {
        return supportedProtocols;
    }

    public void setSupportedProtocols(byte supportedProtocols) {
        this.supportedProtocols = supportedProtocols;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getModel() {
        return model;
    }

    public void setModel(String model) {
        this.model = model;
    }
    
    public String getIpAddress() {
        return ipAddress;
    }

    public void setIpAddress(String ipAddress) {
        this.ipAddress = ipAddress;
    }

    public String getPartner() {
        return partner;
    }

    public void setPartner(String partner) {
        this.partner = partner;
    }

    public Integer getPort() {
        return port;
    }

    public void setPort(Integer port) {
        this.port = port;
    }

    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public String getPassword() {
        return this.password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public boolean getMultistore() {
        return multistore;
    }

    public void setMultistore(boolean multistore) {
        this.multistore = multistore;
    }

    public Integer getOverallStatus() {
        return overallStatus;
    }

    public void setOverallStatus(Integer overallStatus) {
        this.overallStatus = overallStatus;
    }

    public String getOverallStatusReason() {
        return overallStatusReason;
    }

    public void setOverallStatusReason(String overallStatusReason) {
        this.overallStatusReason = overallStatusReason;
    }

    public String getVersion() {
        return version;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    public String getCfMode() {
        return cfMode;
    }

    public void setCfMode(String cfMode) {
        this.cfMode = cfMode;
    }

    public String getCfStatus() {
        return cfStatus;
    }

    public void setCfStatus(String cfStatus) {
        this.cfStatus = cfStatus;
    }

    public Long getTotalCapacity() {
        return totalCapacity;
    }

    public void setTotalCapacity(Long totalCapacity) {
        this.totalCapacity = totalCapacity;
    }

    public Long getTotalAllocated() {
        return totalAllocated;
    }

    public void setTotalAllocated(Long totalAllocated) {
        this.totalAllocated = totalAllocated;
    }

    public Long getTotalUsed() {
        return totalUsed;
    }

    public void setTotalUsed(Long totalUsed) {
        this.totalUsed = totalUsed;
    }

    public Long getTotalFree() {
        return totalFree;
    }

    public void setTotalFree(Long totalFree) {
        this.totalFree = totalFree;
    }

    public String getControllerGroupingName() {
        return controllerGroupingName;
    }

    public void setControllerGroupingName(String controllerGroupingName) {
        this.controllerGroupingName = controllerGroupingName;
    }

    /**
     * @return the isCreateVolumeWrapper
     */
    public boolean getCreateVolumeWrapper() {
        return isCreateVolumeWrapper;
    }

    /**
     * @param isCreateVolumeWrapper
     *            the isCreateVolumeWrapper to set
     */
    public void setCreateVolumeWrapper(boolean isCreateVolumeWrapper) {
        this.isCreateVolumeWrapper = isCreateVolumeWrapper;
    }

    /**
     * @return the isSpaceReserveNewVolumes
     */
    public boolean getSpaceReserveNewVolumes() {
        return isSpaceReserveNewVolumes;
    }

    /**
     * @param isSpaceReserveNewVolumes
     *            the isSpaceReserveNewVolumes to set
     */
    public void setSpaceReserveNewVolumes(boolean isSpaceReserveNewVolumes) {
        this.isSpaceReserveNewVolumes = isSpaceReserveNewVolumes;
    }

    /**
     * @return the isThinProvisionVolumeClones
     */
    public boolean getThinProvisionVolumeClones() {
        return isThinProvisionVolumeClones;
    }

    /**
     * @param isThinProvisionVolumeClones
     *            the isThinProvisionVolumeClones to set
     */
    public void setThinProvisionVolumeClones(boolean isThinProvisionVolumeClones) {
        this.isThinProvisionVolumeClones = isThinProvisionVolumeClones;
    }

    /**
     * @return the isDeleteVolumeIfLastLun
     */
    public boolean getDeleteVolumeIfLastLun() {
        return isDeleteVolumeIfLastLun;
    }

    /**
     * @param isDeleteVolumeIfLastLun
     *            the isDeleteVolumeIfLastLun to set
     */
    public void setDeleteVolumeIfLastLun(boolean isDeleteVolumeIfLastLun) {
        this.isDeleteVolumeIfLastLun = isDeleteVolumeIfLastLun;
    }

    /**
     * @return the volumeWrapperBuffer
     */
    public int getVolumeWrapperBuffer() {
        return volumeWrapperBuffer;
    }

    /**
     * @param volumeWrapperBuffer
     *            the volumeWrapperBuffer to set
     */
    public void setVolumeWrapperBuffer(int volumeWrapperBuffer) {
        this.volumeWrapperBuffer = volumeWrapperBuffer;
    }
    
    public Map<String, Volume> getVolumes() {
        return volumes;
    }

    public void setVolumes(Map<String, Volume> volumes) {
        this.volumes = volumes;
    }

    public Map<String, Lun> getLuns() {
        return luns;
    }

    public void setLuns(Map<String, Lun> luns) {
        this.luns = luns;
    }

    public List<Aggregate> getAggregate() {
        return aggregate;
    }

    public void setAggregate(List<Aggregate> aggregate) {
        this.aggregate = aggregate;
    }

    public List<Adapter> getAdapter() {
        return adapter;
    }

    public void setAdapter(List<Adapter> adapter) {
        this.adapter = adapter;
    }

    public Map<String, Qtree> getQtree() {
        return qtree;
    }

    public void setQtree(Map<String, Qtree> qtree) {
        this.qtree = qtree;
    }

    public Set<String> getInterfaces() {
        return interfaces;
    }

    public void setInterfaces(Set<String> interfaces) {
        this.interfaces = interfaces;
    }


    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder();
        builder.append("Controller [");
        builder.append(", name=");
        builder.append(name);
        builder.append(", model=");
        builder.append(model);
        builder.append(", ipAddress=");
        builder.append(ipAddress);
        builder.append(", partner=");
        builder.append(partner);
        builder.append(", port=");
        builder.append(port);
        builder.append(", username=");
        builder.append(username);
        builder.append(", password=*****");
        builder.append(", useSSL=");
        builder.append(useSSL);
        builder.append(", multistore=");
        builder.append(multistore);
        builder.append(", overallStatus=");
        builder.append(overallStatus);
        builder.append(", overallStatusReason=");
        builder.append(overallStatusReason);
        builder.append(", version=");
        builder.append(version);
        builder.append(", cfMode=");
        builder.append(cfMode);
        builder.append(", cfStatus=");
        builder.append(cfStatus);
        builder.append(", totalCapacity=");
        builder.append(totalCapacity);
        builder.append(", totalAllocated=");
        builder.append(totalAllocated);
        builder.append(", totalUsed=");
        builder.append(totalUsed);
        builder.append(", totalFree=");
        builder.append(totalFree);
        builder.append(", vaaiCapable=");
        builder.append(vaaiCapable);
        builder.append(", supportedProtocols=");
        builder.append(supportedProtocols);
        builder.append(", flagSet=");
        builder.append(flagSet);
        builder.append(", type=");
        builder.append(type);
        builder.append(", ipAddressList= ");
        if (ipAddressList != null && !ipAddressList.isEmpty()) {
            for (String ip : ipAddressList) {
                builder.append("ip[").append(ip).append("],");
            }
        }
        builder.append(", availableInterfaceNames=");
        builder.append(availableInterfaceNames);
        builder.append(", usableInterfaceNames=");
        builder.append(usableInterfaceNames);
        builder.append(", availableVolumeNames=");
        builder.append(availableVolumeNames);
        builder.append(", usableVolumeNames=");
        builder.append(usableVolumeNames);
        builder.append(", availableAggregateNames=");
        builder.append(availableAggregateNames);
        builder.append(", usableAggregateNames=");
        builder.append(usableAggregateNames);

        builder.append(", isCreateVolumeWrapper=");
        builder.append(isCreateVolumeWrapper);
        builder.append(", isSpaceReserveNewVolumes=");
        builder.append(isSpaceReserveNewVolumes);
        builder.append(", isThinProvisionVolumeClones=");
        builder.append(isThinProvisionVolumeClones);
        builder.append(", isDeleteVolumeIfLastLun=");
        builder.append(isDeleteVolumeIfLastLun);
        builder.append(", volumeWrapperBuffer=");
        builder.append(volumeWrapperBuffer);
        builder.append(super.toString());
        builder.append("]");
        return builder.toString();
    }
    
    @Override
    public int hashCode() {
        return getFormattedURIString(getId()).hashCode();
    }
    
    @Override
    public boolean equals(Object other) {
        if (other instanceof Controller) {
            Controller cOther = (Controller) other;
            return cOther.getId().equals(getId()) && cOther.getFormattedURIString(cOther.getId()).equals(getFormattedURIString(getId()));
        }
        return false;
    }

}
