/*
 * $Id: //depot/prod/nvpf/Rinxs/src/java/com/netapp/nvpf/api/model/virtualmachine/migration/MigrationModel.java#1 $
 * 
 * Copyright (c) 2013 NetApp, Inc.
 * All rights reserved.
 * 
 */
package com.netapp.nvpf.api.model.virtualmachine.migration;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import com.netapp.nvpf.api.model.ModelObject;
import com.netapp.nvpf.api.model.datastore.Datastore;
import com.netapp.nvpf.api.model.datastore.provisioning.ProvisioningErrorEnum;
import com.netapp.nvpf.api.model.datastore.provisioning.ProvisioningErrorMessage;
import com.netapp.nvpf.api.model.datastore.provisioning.ProvisioningMessage;
import com.netapp.nvpf.api.model.datastore.provisioning.ProvisioningTargetController;
import com.netapp.nvpf.api.model.datastore.provisioning.ProvisioningTargetControllerDetail;
import com.netapp.nvpf.api.model.datastore.provisioning.ProvisioningTargetStoragePod;
import com.netapp.nvpf.api.model.datastore.provisioning.ProvisioningWarningEnum;
import com.netapp.nvpf.api.model.datastore.provisioning.ProvisioningWarningMessage;

public class MigrationModel extends ModelObject {

    private static final long serialVersionUID = 2010307013874058143L;
    
    private static final int DEFAULT_DATASTORE_GROW_SIZE_MBS = (int) convertGBToMB(100);
    private static final int DEFAULT_GROW_INCREMENT_MBS = (int) convertGBToMB(1);
    private static final int VM_METADATA_SIZE_BYTES = (int)convertGBToByte(1);
    
    public static final int MAX_NUMBER_OF_VMFS_DATASTORES = 256;
    public static final int MAX_DATASTORE_NAME_LENGTH = 42;
    public static final int MIN_DATASTORE_SIZE_BYTES = 1073741824;
    public static final String NFS = "NFS";
    public static final String VMFS = "VMFS";
    public static final String FCP = "FC/FCoE";
    public static final String ISCSI = "iSCSI";
       
    private ScannedVirtualMachine[] vmsToAlign;
    private MigrationHost[] qualifiedHosts;
    private Datastore[] availableDatastores;
    private ProvisioningTargetController provisioningTargetController;
    private ProvisioningTargetControllerDetail provisioningTargetControllerDetail;
    private ProvisioningTargetStoragePod provisioningTargetStoragePod;
    
    private Datastore datastore;
    private String datastoreType;
    private Integer offset;
    private boolean isVFilerContext;
    private String vfilerContext;
    private String alignmentType;
    private String storagePath;
    private String datastoreName;
    private boolean createVolumeWrapper;
    private boolean thinProvision;
    private boolean autoGrow;
    private double growIncrement;
    private double maxGrowSize;
    private String blockSize;
    private boolean areDestinationsAllVSphere5;
    private String vcontrollerId;
    private String datastoreProtocol;   
    private String aggrOrVolName;
    private double datastoreSize;
    private double minDatastoreSizeGB;
    private Integer blockSizeForSpec;
    private ProvisioningMessage[] hostErrors;
    private ProvisioningMessage[] nfsLicensingErrors;
    private ProvisioningMessage[] fcpLicensingErrors;
    private ProvisioningMessage[] iscsiLicensingErrors;
    private HashMap<String, ScannedVirtualMachine[]> datastoreVMMap;
    
    public MigrationModel() {
        setDatastoreType("NFS");
        setAlignmentType("ONLINE_ALIGNMENT");
        setStoragePath("EXISTING_DATASTORE");
        setDatastoreName("newDatastore");
        setCreateVolumeWrapper(false);
        setThinProvision(false);
        setAutoGrow(false);
        setGrowIncrement(DEFAULT_GROW_INCREMENT_MBS);
        setMaxGrowSize(DEFAULT_DATASTORE_GROW_SIZE_MBS);
        setBlockSize("Default");
    }
    
    public MigrationModel(ScannedVirtualMachine[] vmsToAlign) {
        this.vmsToAlign = vmsToAlign;
        
        setOffset(vmsToAlign[0].getOffset());
        setDatastoreType("NFS");
        setAlignmentType("ONLINE_ALIGNMENT");
        setStoragePath("EXISTING_DATASTORE");
        setDatastoreName("newDatastore");
        setCreateVolumeWrapper(false);
        setThinProvision(false);
        setAutoGrow(false);
        setGrowIncrement(DEFAULT_GROW_INCREMENT_MBS);
        setMaxGrowSize(DEFAULT_DATASTORE_GROW_SIZE_MBS);
        setBlockSize("Default");
    }
    
    private static double convertGBToMB(double val) {
        return val * 1024L;
    }
    
    private static double convertGBToByte(double val) {
        return convertGBToMB(val) * 1024L * 1024L;
    }
    
    private static double convertMBToGB(double val) {
        return val / 1024L;
    }
    
    private static double convertByteToMB(double val) {
        return ((val / 1024) / 1024);
    }
    
    private static double convertByteToGB(double val) {
        return convertMBToGB(convertByteToMB(val));
    }

    public ScannedVirtualMachine[] getVmsToAlign() {
        return vmsToAlign;
    }


    public void setVmsToAlign(ScannedVirtualMachine[] vmsToAlign) {
        this.vmsToAlign = vmsToAlign;
    }


    public MigrationHost[] getQualifiedHosts() {
        return qualifiedHosts;
    }

    public Datastore getDatastore() {
        return datastore;
    }

    public void setDatastore(Datastore datastore) {
        this.datastore = datastore;
    }

    public String getVcontrollerId() {
        return vcontrollerId;
    }

    public void setVcontrollerId(String vcontrollerId) {
        this.vcontrollerId = vcontrollerId;
    }

    public boolean isVFilerContext() {
        return isVFilerContext;
    }

    public void setVFilerContext(boolean isVFilerContext) {
        this.isVFilerContext = isVFilerContext;
    }

    public String getVfilerContext() {
        return vfilerContext;
    }

    public void setVfilerContext(String vfilerContext) {
        this.vfilerContext = vfilerContext;
    }

    public void setQualifiedHosts(MigrationHost[] qualifiedHosts) {
        this.qualifiedHosts = qualifiedHosts;
        getLicensingErrors();
    }

    public Datastore[] getAvailableDatastores() {
        return availableDatastores;
    }


    public void setAvailableDatastores(Datastore[] availableDatastores) {
        this.availableDatastores = availableDatastores;
    }

    public boolean isAreDestinationsAllVSphere5() {
        for (MigrationHost host : getQualifiedHosts()) {
            if (host.getVersion() == null || !host.getVersion().equalsIgnoreCase("VERSION_5")) {
                areDestinationsAllVSphere5 = false;
                return areDestinationsAllVSphere5;
            }
        }
        areDestinationsAllVSphere5 = true;
        return areDestinationsAllVSphere5;
    }

    public void setAreDestinationsAllVSphere5(boolean areDestinationsAllVSphere5) {
        this.areDestinationsAllVSphere5 = areDestinationsAllVSphere5;
    }

    public String getDatastoreType() {
        return datastoreType;
    }

    public void setDatastoreType(String dataStoreType) {
        this.datastoreType = dataStoreType;
    }

    public Integer getOffset() {
        return offset;
    }

    public void setOffset(Integer offset) {
        this.offset = offset;
    }

    public String getAlignmentType() {
        return alignmentType;
    }

    public void setAlignmentType(String alignmentType) {
        this.alignmentType = alignmentType;
    }

    public String getStoragePath() {
        return storagePath;
    }

    public void setStoragePath(String storagePath) {
        this.storagePath = storagePath;
    }

    public String getDatastoreName() {
        return datastoreName;
    }

    public void setDatastoreName(String datastoreName) {
        this.datastoreName = datastoreName;
    }

    public boolean isCreateVolumeWrapper() {
        return createVolumeWrapper;
    }

    public void setCreateVolumeWrapper(boolean createVolumeWrapper) {
        this.createVolumeWrapper = createVolumeWrapper;
    }

    public boolean isThinProvision() {
        return thinProvision;
    }

    public void setThinProvision(boolean thinProvision) {
        this.thinProvision = thinProvision;
    }

    public boolean isAutoGrow() {
        return autoGrow;
    }

    public void setAutoGrow(boolean autoGrow) {
        this.autoGrow = autoGrow;
    }

    public double getGrowIncrement() {
        return growIncrement;
    }

    public void setGrowIncrement(double growIncrement) {
        this.growIncrement = growIncrement;
    }

    public double getMaxGrowSize() {
        return maxGrowSize;
    }

    public void setMaxGrowSize(double maxGrowSize) {
        this.maxGrowSize = maxGrowSize;
    }

    public String getBlockSize() {
        return blockSize;
    }

    public void setBlockSize(String blockSize) {
        this.blockSize = blockSize;
    }

    public ProvisioningTargetController getProvisioningTargetController() {
        return provisioningTargetController;
    }

    public void setProvisioningTargetController(
            ProvisioningTargetController provisioningTargetController) {
        this.provisioningTargetController = provisioningTargetController;
    }

    public ProvisioningTargetControllerDetail getProvisioningTargetControllerDetail() {
        return provisioningTargetControllerDetail;
    }

    public void setProvisioningTargetControllerDetail(
            ProvisioningTargetControllerDetail provisioningTargetControllerDetail) {
        this.provisioningTargetControllerDetail = provisioningTargetControllerDetail;
    }

    public ProvisioningTargetStoragePod getProvisioningTargetStoragePod() {
        if (provisioningTargetStoragePod == null) {
            provisioningTargetStoragePod = new ProvisioningTargetStoragePod();
        }
        return provisioningTargetStoragePod;
    }

    public void setProvisioningTargetStoragePod(
            ProvisioningTargetStoragePod provisioningTargetStoragePod) {
        this.provisioningTargetStoragePod = provisioningTargetStoragePod;
    }

    public String getDatastoreProtocol() {
        return datastoreProtocol;
    }

    public void setDatastoreProtocol(String datastoreProtocol) {
        this.datastoreProtocol = datastoreProtocol;
    }
    
    public ProvisioningMessage[] getHostErrors() {
        return hostErrors;
    }

    public void setHostErrors(ProvisioningMessage[] hostErrors) {
        this.hostErrors = hostErrors;
    }

    public ProvisioningMessage[] getNfsLicensingErrors() {
        return nfsLicensingErrors;
    }

    public void setNfsLicensingErrors(ProvisioningMessage[] nfsLicensingErrors) {
        this.nfsLicensingErrors = nfsLicensingErrors;
    }

    public ProvisioningMessage[] getFcpLicensingErrors() {
        return fcpLicensingErrors;
    }

    public void setFcpLicensingErrors(ProvisioningMessage[] fcpLicensingErrors) {
        this.fcpLicensingErrors = fcpLicensingErrors;
    }

    public ProvisioningMessage[] getIscsiLicensingErrors() {
        return iscsiLicensingErrors;
    }

    public void setIscsiLicensingErrors(ProvisioningMessage[] iscsiLicensingErrors) {
        this.iscsiLicensingErrors = iscsiLicensingErrors;
    }

    private void getLicensingErrors() {

        List<ProvisioningMessage> hostError = new ArrayList<ProvisioningMessage>();
        List<ProvisioningMessage> nfsLicensingError = new ArrayList<ProvisioningMessage>();
        List<ProvisioningMessage> fcpLicensingError = new ArrayList<ProvisioningMessage>();
        List<ProvisioningMessage> iscsiLicensingError = new ArrayList<ProvisioningMessage>();
        
        /**
         * If for selected datacenter/host no items are found then add an error that no qualified host error.
         */
        if (qualifiedHosts == null || qualifiedHosts.length == 0) {
            hostError.add(new ProvisioningErrorMessage(ProvisioningErrorEnum.NO_QUALIFIED_HOSTS_ERROR));
            if (this.hostErrors != null) {
                this.hostErrors = hostError.toArray(this.hostErrors);
            }
            return;
        }

        int maxAllowedNfsDatastores = 256;
        int maxDatastores = MAX_NUMBER_OF_VMFS_DATASTORES;

        int currentDatastores = 0;

        for (MigrationHost host : qualifiedHosts) {

            if (host.getPowerState().equalsIgnoreCase("standby")) {
                hostError.add(new ProvisioningErrorMessage(ProvisioningErrorEnum.HOST_IN_STAND_BY_ERROR, host.getName()));
                continue;
            }

            if (host.isInMantMode()) {
                hostError.add(new ProvisioningErrorMessage(ProvisioningErrorEnum.HOST_IN_MAINTENANCE_MODE_ERROR, host.getName()));
                continue;
            }

            if (!host.isNfsSupported()) {
                nfsLicensingError.add(new ProvisioningWarningMessage(ProvisioningWarningEnum.HOST_TYPE_SUPPORTED_WARNING, host.getName(), NFS));
            }

            if (!host.isFcpSupported()) {
                fcpLicensingError.add(new ProvisioningWarningMessage(ProvisioningWarningEnum.HOST_TYPE_SUPPORTED_WARNING, host.getName(), FCP));
            }
            if (!host.isIscsiSupported()) {
                iscsiLicensingError.add(new ProvisioningWarningMessage(ProvisioningWarningEnum.HOST_TYPE_SUPPORTED_WARNING, host.getName(), ISCSI));
            }

            int allowedNfsDatastores = host.getMaxNfsDatastores() - host.getNfsDatastoreCount();
            if (allowedNfsDatastores < maxAllowedNfsDatastores) {
                maxAllowedNfsDatastores = allowedNfsDatastores;
            }

            if (host.getVmfsDatastoreCount() > currentDatastores) {
                currentDatastores = host.getVmfsDatastoreCount();
            }
        }
        /**
         * To check whether newNFSDatastore allowed or not
         */
        if (maxAllowedNfsDatastores <= 0) {
            nfsLicensingError.add(new ProvisioningErrorMessage(ProvisioningErrorEnum.MAX_NFS_DATASTORES_EXCEEDED));
        }

        /**
         * To check whether newVMFSDatastore allowed or not
         */
        if ((maxDatastores - currentDatastores) <= 0) {
            fcpLicensingError.add(new ProvisioningErrorMessage(ProvisioningErrorEnum.MAX_VMFS_DATASTORES_EXCEEDED));
            iscsiLicensingError.add(new ProvisioningErrorMessage(ProvisioningErrorEnum.MAX_VMFS_DATASTORES_EXCEEDED));
        }

        this.hostErrors = new ProvisioningMessage[hostError.size()];
        this.hostErrors = hostError.toArray(this.hostErrors);
        
        this.nfsLicensingErrors = new ProvisioningMessage[nfsLicensingError.size()];
        this.nfsLicensingErrors = nfsLicensingError.toArray(this.nfsLicensingErrors);
        
        this.fcpLicensingErrors = new ProvisioningMessage[fcpLicensingError.size()];
        this.fcpLicensingErrors = fcpLicensingError.toArray(this.fcpLicensingErrors);
        
        this.iscsiLicensingErrors = new ProvisioningMessage[iscsiLicensingError.size()];
        this.iscsiLicensingErrors = iscsiLicensingError.toArray(this.iscsiLicensingErrors);
    }

    public String getAggrOrVolName() {
        return aggrOrVolName;
    }

    public void setAggrOrVolName(String aggrOrVolName) {
        this.aggrOrVolName = aggrOrVolName;
    }
    
    public String getOptimizedDatastoreName() {
        final String baseName = getDatastoreName();
        final Integer localOffset = getOffset();
        if (baseName != null && localOffset != null && localOffset != 0 && getDatastoreType().equalsIgnoreCase("VMFS")) {
            return baseName + "_optimized_" + localOffset;
        }
        else if (baseName != null && offset != null && offset != 0 && getDatastoreType().equalsIgnoreCase("NFS")) {
            return baseName + "_optimized_" + localOffset; // could probably just take out the check for the type, but this works
        }
        else {
            return baseName;
        }
    }

    public double getDatastoreSize() {
        if (this.datastoreSize == 0) {
            setDatastoreSize(Math.ceil(getMinDatastoreSize() / 1024.0) * 1024.0);
            // (round up to the nearest GB)
        }
        return this.datastoreSize;
    }

    public void setDatastoreSize(double datastoreSize) {
        this.datastoreSize = datastoreSize;
    }
    
    /**
     * Returns the minimum datastore size allowed (in MBs)
     * 
     * @return the minimum datastore size allowed (in MBs)
     */
    private double getMinDatastoreSize() {
        long minDatastoreSize = 0;
        
        for (ScannedVirtualMachine vm : getVmsToAlign()) {
            minDatastoreSize += vm.getCommittedBytes();
        }
        
        // add some overhead for the config files
        minDatastoreSize += VM_METADATA_SIZE_BYTES;
        
        // now add 10% buffer so that the LUN doesn't go offline (BURT 537471)
        minDatastoreSize += (minDatastoreSize / 10L);
        
        return convertByteToMB(minDatastoreSize);
    }

    public double getMinDatastoreSizeGB() {
        this.minDatastoreSizeGB = convertMBToGB(getMinDatastoreSize());
        return this.minDatastoreSizeGB;
    }

    public void setMinDatastoreSizeGB(double minDatastoreSizeGB) {
        this.minDatastoreSizeGB = minDatastoreSizeGB;
    }

    public Integer getBlockSizeForSpec() {
        
        /**
         * Valid block sizes for VMFS3 are 1, 2, 4, and 8.
         * As flex is not able to set null , we manually returning null incase value is 0
         */
        if (blockSizeForSpec == 0) {
            return null;
        }
        return blockSizeForSpec;
    }

    public void setBlockSizeForSpec(Integer blockSizeForSpec) {
        this.blockSizeForSpec = blockSizeForSpec;
    }

    public HashMap<String, ScannedVirtualMachine[]> getDatastoreVMMap() {
        return datastoreVMMap;
    }

    public void setDatastoreVMMap(HashMap<String, ScannedVirtualMachine[]> datastoreVMMap) {
        this.datastoreVMMap = datastoreVMMap;
    }

}
