/*
* $Id: //depot/prod/kamino/main/util/alignment/src/cpp/VirtualDiskInfo.cpp#7 $
*
* Copyright (c) 2010 NetApp, Inc.
* All rights reserved.
*/

#include <windows.h>
#include <string>

#include "vixDiskLib.h"
#include "VixDiskHelper.h"
#include "VirtualDiskInfo.h"

using std::string;

static void DumpBytes(const uint8 *buf, size_t n, int step);

/*
 * Class:     com_netapp_alignment_virtualDisk_VirtualDiskInfo
 * Method:    readSectors
 * Signature: (Ljava/lang/String;ILjava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;JJ)[I
 */
JNIEXPORT jintArray JNICALL Java_com_netapp_alignment_virtualDisk_VirtualDiskInfo_readSectors
(JNIEnv *env, jclass joClass, jstring jhost, jint jcredType, jstring jusername, jstring jpassword, 
    jstring jvmxSpec, jstring jdiskPath, jlong jstart, jlong jcount) {
	
	if(jstart < 0 || jcount < 0) {
		env->ThrowNew(env->FindClass("java/lang/Exception"), "Parameters start and count must be >= 0");
		return NULL;
	}
	
	VixError vixError = VixDiskLib_Init(VIXDISKLIB_VERSION_MAJOR,
										VIXDISKLIB_VERSION_MINOR,
										NULL, NULL, NULL, NULL);

	if(VIX_FAILED(vixError)) {
		char* msg = VixDiskLib_GetErrorText(vixError, NULL);
		env->ThrowNew(env->FindClass("java/lang/Exception"), msg);
		VixDiskLib_FreeErrorText(msg);
		return NULL;
	}

	char* host = (char*)env->GetStringUTFChars(jhost, NULL);
	char* username = (char*)env->GetStringUTFChars(jusername, NULL);
	char* password = (char*)env->GetStringUTFChars(jpassword, NULL);
	char* vmxSpec = (char*)env->GetStringUTFChars(jvmxSpec, NULL);

	// build the connection parameters
	VixDiskLibConnectParams cnxParams = {0};
	cnxParams.serverName = host;
	cnxParams.port = 902; // default port

	if(jcredType == VIXDISKLIB_CRED_UID) {
		cnxParams.credType = VIXDISKLIB_CRED_UID;
	} else {
		cnxParams.credType = VIXDISKLIB_CRED_SSPI;
	}

	cnxParams.creds.uid.userName = username;
	cnxParams.creds.uid.password = password;
	cnxParams.vmxSpec = vmxSpec;

    VixDiskLibConnection connection;
    vixError = VixDiskLib_Connect(&cnxParams,
							      &connection);

	// clean up
	env->ReleaseStringUTFChars(jhost, host);
	env->ReleaseStringUTFChars(jusername, username);
	env->ReleaseStringUTFChars(jpassword, password);
	env->ReleaseStringUTFChars(jvmxSpec, vmxSpec);

	if(VIX_FAILED(vixError)) {
		char* msg = VixDiskLib_GetErrorText(vixError, NULL);
		env->ThrowNew(env->FindClass("java/lang/Exception"), msg);
		VixDiskLib_FreeErrorText(msg);
		return NULL;
	}

	char* diskPath = (char*)env->GetStringUTFChars(jdiskPath, NULL);

	int bytesToProcess = VIXDISKLIB_SECTOR_SIZE * jcount;
	uint8* buffer = new uint8[bytesToProcess];

    VixDisk disk(connection, diskPath, VIXDISKLIB_FLAG_OPEN_READ_ONLY);

    vixError = VixDiskLib_Read(disk.Handle(),
                               jstart,
                               jcount,
                               buffer);

	env->ReleaseStringUTFChars(jdiskPath, diskPath);

	VixDiskLib_Disconnect(connection);
	VixDiskLib_Exit();

	if(VIX_FAILED(vixError)) {
		char* msg = VixDiskLib_GetErrorText(vixError, NULL);
		env->ThrowNew(env->FindClass("java/lang/Exception"), msg);
		VixDiskLib_FreeErrorText(msg);
		return NULL;
	}

	if(DEBUG) {
		DumpBytes(buffer, VIXDISKLIB_SECTOR_SIZE * jcount, 16);
	}

	jint* arrayToReturn = new jint[bytesToProcess];
	for(int i = 0; i < bytesToProcess; i++) {
		arrayToReturn[i] = buffer[i];
	}

	jintArray sector = env->NewIntArray(bytesToProcess);
	env->SetIntArrayRegion(sector, 0, bytesToProcess, arrayToReturn);

	delete[] buffer;
	delete[] arrayToReturn;

	return sector;
}

/*
 * Class:     com_netapp_alignment_virtualDisk_VirtualDiskInfo
 * Method:    setToolsVersion
 * Signature: (Ljava/lang/String;ILjava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_com_netapp_alignment_virtualDisk_VirtualDiskInfo_setToolsVersion
(JNIEnv *env, jclass joClass, jstring jhost, jint jcredType, jstring jusername, jstring jpassword, 
    jstring jvmxSpec, jstring jdiskPath, jstring jtoolsVersion) {

	VixError vixError = VixDiskLib_Init(VIXDISKLIB_VERSION_MAJOR,
										VIXDISKLIB_VERSION_MINOR,
										NULL, NULL, NULL, NULL);

	if(VIX_FAILED(vixError)) {
		char* msg = VixDiskLib_GetErrorText(vixError, NULL);
		env->ThrowNew(env->FindClass("java/lang/Exception"), msg);
		VixDiskLib_FreeErrorText(msg);
		return;
	}

	char* host = (char*)env->GetStringUTFChars(jhost, NULL);
	char* username = (char*)env->GetStringUTFChars(jusername, NULL);
	char* password = (char*)env->GetStringUTFChars(jpassword, NULL);
	char* vmxSpec = (char*)env->GetStringUTFChars(jvmxSpec, NULL);

	// build the connection parameters
	VixDiskLibConnectParams cnxParams = {0};
	cnxParams.serverName = host;
	cnxParams.port = 902; // default port

	if(jcredType == VIXDISKLIB_CRED_UID) {
		cnxParams.credType = VIXDISKLIB_CRED_UID;
	} else {
		cnxParams.credType = VIXDISKLIB_CRED_SSPI;
	}

	cnxParams.creds.uid.userName = username;
	cnxParams.creds.uid.password = password;
	cnxParams.vmxSpec = vmxSpec;

    VixDiskLibConnection connection;
    vixError = VixDiskLib_Connect(&cnxParams,
							      &connection);

	// clean up
	env->ReleaseStringUTFChars(jhost, host);
	env->ReleaseStringUTFChars(jusername, username);
	env->ReleaseStringUTFChars(jpassword, password);
	env->ReleaseStringUTFChars(jvmxSpec, vmxSpec);

	if(VIX_FAILED(vixError)) {
		char* msg = VixDiskLib_GetErrorText(vixError, NULL);
		env->ThrowNew(env->FindClass("java/lang/Exception"), msg);
		VixDiskLib_FreeErrorText(msg);
		return;
	}

	char* diskPath = (char*)env->GetStringUTFChars(jdiskPath, NULL);
	char* toolsVersion = (char*)env->GetStringUTFChars(jtoolsVersion, NULL);

    VixDisk disk(connection, diskPath, 0);

    vixError = VixDiskLib_WriteMetadata(disk.Handle(),
										"toolsVersion",
                                        toolsVersion);

	env->ReleaseStringUTFChars(jdiskPath, diskPath);
	env->ReleaseStringUTFChars(jtoolsVersion, toolsVersion);

	VixDiskLib_Disconnect(connection);
	VixDiskLib_Exit();

	if(VIX_FAILED(vixError)) {
		char* msg = VixDiskLib_GetErrorText(vixError, NULL);
		env->ThrowNew(env->FindClass("java/lang/Exception"), msg);
		VixDiskLib_FreeErrorText(msg);
		return;
	}
}

/*
 * Class:     com_netapp_alignment_virtualDisk_VirtualDiskInfo
 * Method:    getGeometry
 * Signature: (Ljava/lang/String;ILjava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)Lcom/netapp/alignment/virtualDisk/Geometry;
 */
JNIEXPORT jobject JNICALL Java_com_netapp_alignment_virtualDisk_VirtualDiskInfo_getGeometry
(JNIEnv *env, jclass joClass, jstring jhost, jint jcredType, jstring jusername, jstring jpassword,
    jstring jvmxSpec, jstring jdiskPath) {

	VixError vixError = VixDiskLib_Init(VIXDISKLIB_VERSION_MAJOR,
										VIXDISKLIB_VERSION_MINOR,
										NULL, NULL, NULL, NULL);

	if(VIX_FAILED(vixError)) {
		char* msg = VixDiskLib_GetErrorText(vixError, NULL);
		env->ThrowNew(env->FindClass("java/lang/Exception"), msg);
		VixDiskLib_FreeErrorText(msg);
		return NULL;
	}

	char* host = (char*)env->GetStringUTFChars(jhost, NULL);
	char* username = (char*)env->GetStringUTFChars(jusername, NULL);
	char* password = (char*)env->GetStringUTFChars(jpassword, NULL);
	char* vmxSpec = (char*)env->GetStringUTFChars(jvmxSpec, NULL);

	// build the connection parameters
	VixDiskLibConnectParams cnxParams = {0};
	cnxParams.serverName = host;
	cnxParams.port = 902; // default port

	if(jcredType == VIXDISKLIB_CRED_UID) {
		cnxParams.credType = VIXDISKLIB_CRED_UID;
	} else {
		cnxParams.credType = VIXDISKLIB_CRED_SSPI;
	}

	cnxParams.creds.uid.userName = username;
	cnxParams.creds.uid.password = password;
	cnxParams.vmxSpec = vmxSpec;

    VixDiskLibConnection connection;
    vixError = VixDiskLib_Connect(&cnxParams,
							      &connection);

	// clean up
	env->ReleaseStringUTFChars(jhost, host);
	env->ReleaseStringUTFChars(jusername, username);
	env->ReleaseStringUTFChars(jpassword, password);
	env->ReleaseStringUTFChars(jvmxSpec, vmxSpec);

	if(VIX_FAILED(vixError)) {
		char* msg = VixDiskLib_GetErrorText(vixError, NULL);
		env->ThrowNew(env->FindClass("java/lang/Exception"), msg);
		VixDiskLib_FreeErrorText(msg);
		return NULL;
	}

	char* diskPath = (char*)env->GetStringUTFChars(jdiskPath, NULL);

	VixDisk disk(connection, diskPath, VIXDISKLIB_FLAG_OPEN_READ_ONLY);
	VixDiskLibInfo *info = NULL;
	vixError = VixDiskLib_GetInfo(disk.Handle(), &info);

	env->ReleaseStringUTFChars(jdiskPath, diskPath);

	if(VIX_FAILED(vixError)) {
		char* msg = VixDiskLib_GetErrorText(vixError, NULL);
		env->ThrowNew(env->FindClass("java/lang/Exception"), msg);
		
		VixDiskLib_FreeErrorText(msg);
		VixDiskLib_Disconnect(connection);
		VixDiskLib_Exit();

		return NULL;
	}

	// build the Geometry object
    jclass clazz = env->FindClass("com/netapp/alignment/virtualDisk/Geometry");
    jmethodID constructor = env->GetMethodID(clazz, "<init>", "()V");
    jobject jgeometry = env->NewObject(clazz, constructor);

    jfieldID fid = env->GetFieldID(clazz, "cylinders", "I");
	env->SetIntField(jgeometry, fid, info->physGeo.cylinders);

    fid = env->GetFieldID(clazz, "heads", "I");
	env->SetIntField(jgeometry, fid, info->physGeo.heads);

    fid = env->GetFieldID(clazz, "sectors", "I");
	env->SetIntField(jgeometry, fid, info->physGeo.sectors);

	VixDiskLib_FreeInfo(info);

	VixDiskLib_Disconnect(connection);
	VixDiskLib_Exit();

	return jgeometry;
}

static void DumpBytes(const unsigned char *buf, size_t n, int step)
{
	printf("DumpBytes\n");

	size_t lines = n / step;
	size_t i;

	printf("Lines: %d\n", lines);

	for (i = 0; i < lines; i++) {
		int k, last;
		printf("%04"FMTSZ"x : ", i * step);
		for (k = 0; n != 0 && k < step; k++, n--) {
			printf("%02x ", buf[i * step + k]);
		}
		printf("  ");
		last = k;
		while (k --) {
			unsigned char c = buf[i * step + last - k - 1];
			if (c < ' ' || c >= 127) {
				c = '.';
			}
			printf("%c", c);
		}
		printf("\n");
	}

	printf("\n");
}
