/*
 * $Id: //depot/prod/kamino/main/util/alignment/src/java/com/netapp/alignment/virtualDisk/VirtualDiskInfo.java#6 $
 *
 * Copyright (c) 2010 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.alignment.virtualDisk;

public class VirtualDiskInfo {
    static {
        // load all dependent libraries
        System.loadLibrary("iconv");
        System.loadLibrary("intl");
        System.loadLibrary("glib-2.0");
        System.loadLibrary("gobject-2.0");
        System.loadLibrary("libeay32");
        System.loadLibrary("ssleay32");
        System.loadLibrary("liblber");
        System.loadLibrary("libldap_r");
        System.loadLibrary("libcurl");
        System.loadLibrary("zlib1");
        System.loadLibrary("libxml2");
        System.loadLibrary("gvmomi");
        System.loadLibrary("vixDiskLibVim");
        System.loadLibrary("vixDiskLib");

        // load our library
        System.loadLibrary("virtualDiskInfo");
    }

    private String host;

    private String username;
    private String password;

    private CredentialType credType;
    
    enum CredentialType {
        UID, 
        SESSION_ID, 
        TICKET_ID, 
        SSPI;
    }

    /**
     * Builds a VirtualDiskInfo object using the VC host. This constructor will
     * use SSPI authentication to connect to the vCenter server. SSPI assumes
     * the currently logged on user has the same credentials as a user on the
     * vCenter server or is on the domain.
     * 
     * @param host
     *            The host name or IP address of the vCenter server
     */
    public VirtualDiskInfo(String host) {
        this.host = host;
        
        this.credType = CredentialType.SSPI;
        this.username = "__sspi__";
        this.password = "dummy"; // this can be anything but null
    }

    /**
     * Builds a VirtualDiskInfo object using the VC host, username, and password
     * 
     * @param host
     *            The host name or IP address of the vCenter server
     * @param username
     *            The username used to connect to the vCenter server
     * @param password
     *            The password used to connect to the vCenter server
     */
    public VirtualDiskInfo(String host, String username, String password) {
        this.host = host;
        this.username = username;
        this.password = password;

        this.credType = CredentialType.UID;
    }

    /**
     * Returns the specified sectors into a byte[]
     * 
     * @param vmxSpec
     *            A specification defining the VMX path of the virtual machine
     *            to read. The VMX path is of the form <VM name>/<VMX
     *            name>?dcPath=<Datacenter name>&dsName=<Datastore name>
     * @param diskPath
     *            The path to the virtual disk to be read. The disk path is of
     *            the form [<Datastore name>] <VM name>/<VMDK name>
     * @param start
     *            The sector to read
     * @param count
     *            The number of sectors to read
     * @return the specified sector in an int array
     * @throws Exception
     */
    public int[] readSectors(String vmxSpec, String diskPath, long start, long count) throws Exception {
        return readSectors(host, 
                           (credType.ordinal() + 1), // VDDK start numbering this at 1
                           username, 
                           password, 
                           vmxSpec, 
                           diskPath, 
                           start, 
                           count);
    }
    private static native int[] readSectors(String host, int credType, String username, String password, 
            String vmxSpec, String diskPath, long start, long count) throws Exception;
    
    public void setToolsVersion(String vmxSpec, String diskPath, int toolsVersion) throws Exception {
        setToolsVersion(host, 
                        (credType.ordinal() + 1), // VDDK start numbering this at 1
                        username, 
                        password, 
                        vmxSpec, 
                        diskPath, 
                        String.valueOf(toolsVersion));
    }
    private static native void setToolsVersion(String host, int credType, String username, String password,
            String vmxSpec, String diskPath, String toolsVersion) throws Exception;
    
    public Geometry getGeometry(String vmxSpec, String diskPath) throws Exception {
        return getGeometry(host, 
                           (credType.ordinal() + 1), 
                           username, 
                           password,
                           vmxSpec,
                           diskPath);
    }
    private static native Geometry getGeometry(String host, int credType, String username, String password,
            String vmxSpec, String diskPath) throws Exception;
}
